CONFIG = {
    "QtWidgets.QAbstractButton": {
        "addAction": True,
        "addActions": True,
        "animateClick": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hitButton": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoExclusive": True,
        "setAutoFillBackground": True,
        "setAutoRepeat": True,
        "setAutoRepeatDelay": True,
        "setAutoRepeatInterval": True,
        "setBackgroundRole": True,
        "setCheckable": True,
        "setChecked": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setDown": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcut": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QAbstractGraphicsShapeItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setBrush": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPen": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QAbstractItemDelegate": {
        "blockSignals": True,
        "connectNotify": True,
        "createEditor": True,
        "destroyEditor": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "paint": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setEditorData": True,
        "setModelData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sizeHint": True,
        "startTimer": True,
        "tr": True,
        "updateEditorGeometry": True
    },
    "QtWidgets.QAbstractItemView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "closePersistentEditor": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "editorDestroyed": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isPersistentEditorOpen": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "openPersistentEditor": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setRootIndex": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setupViewport": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualRect": True,
        "visualRegionForSelection": True,
        "wheelEvent": True
    },
    "QtWidgets.QAbstractScrollArea": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "dragEnterEvent": True,
        "dragLeaveEvent": True,
        "dragMoveEvent": True,
        "drawFrame": True,
        "dropEvent": True,
        "event": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setupViewport": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "viewportEvent": True,
        "wheelEvent": True
    },
    "QtWidgets.QAbstractSlider": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setInvertedAppearance": True,
        "setInvertedControls": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPageStep": True,
        "setPalette": True,
        "setProperty": True,
        "setRange": True,
        "setRepeatAction": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSingleStep": True,
        "setSliderDown": True,
        "setSliderPosition": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTracking": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "sliderChange": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "triggerAction": True,
        "ungrabGesture": True
    },
    "QtWidgets.QAbstractSpinBox": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAccelerated": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButtonSymbols": True,
        "setContextMenuPolicy": True,
        "setCorrectionMode": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setGroupSeparatorShown": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeyboardTracking": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setReadOnly": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSpecialValueText": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWrapping": True,
        "stackUnder": True,
        "startTimer": True,
        "stepBy": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "validate": True
    },
    "QtWidgets.QAction": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setActionGroup": True,
        "setAutoRepeat": True,
        "setCheckable": True,
        "setChecked": True,
        "setData": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFont": True,
        "setIcon": True,
        "setIconText": True,
        "setIconVisibleInMenu": True,
        "setMenu": True,
        "setMenuRole": True,
        "setObjectName": True,
        "setParent": True,
        "setPriority": True,
        "setProperty": True,
        "setSeparator": True,
        "setShortcut": True,
        "setShortcutContext": True,
        "setShortcutVisibleInContextMenu": True,
        "setStatusTip": True,
        "setText": True,
        "setToolTip": True,
        "setVisible": True,
        "setWhatsThis": True,
        "showStatusText": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QActionGroup": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAction": True,
        "removeEventFilter": True,
        "setDisabled": True,
        "setEnabled": True,
        "setExclusionPolicy": True,
        "setExclusive": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setVisible": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QApplication": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "installNativeEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "notify": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeNativeEventFilter": True,
        "setAutoSipEnabled": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setStyleSheet": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QBoxLayout": {
        "addChildLayout": True,
        "addChildWidget": True,
        "addItem": True,
        "addLayout": True,
        "addSpacerItem": True,
        "addSpacing": True,
        "addStretch": True,
        "addStrut": True,
        "alignmentRect": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "heightForWidth": True,
        "inherits": True,
        "insertItem": True,
        "insertLayout": True,
        "insertSpacerItem": True,
        "insertSpacing": True,
        "insertStretch": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemAt": True,
        "killTimer": True,
        "minimumHeightForWidth": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "removeWidget": True,
        "replaceWidget": True,
        "setDirection": True,
        "setEnabled": True,
        "setGeometry": True,
        "setMenuBar": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSizeConstraint": True,
        "setSpacing": True,
        "setStretch": True,
        "startTimer": True,
        "stretch": True,
        "takeAt": True,
        "totalHeightForWidth": True,
        "tr": True,
        "widgetEvent": True
    },
    "QtWidgets.QButtonGroup": {
        "addButton": True,
        "blockSignals": True,
        "button": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "id": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeButton": True,
        "removeEventFilter": True,
        "setExclusive": True,
        "setId": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QCalendarWidget": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintCell": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCalendar": True,
        "setContextMenuPolicy": True,
        "setCurrentPage": True,
        "setCursor": True,
        "setDateEditAcceptDelay": True,
        "setDateEditEnabled": True,
        "setDateRange": True,
        "setDateTextFormat": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFirstDayOfWeek": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setGridVisible": True,
        "setHeaderTextFormat": True,
        "setHidden": True,
        "setHorizontalHeaderFormat": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumDate": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumDate": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setNavigationBarVisible": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setSelectedDate": True,
        "setSelectionMode": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalHeaderFormat": True,
        "setVisible": True,
        "setWeekdayTextFormat": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "updateCell": True,
        "weekdayTextFormat": True
    },
    "QtWidgets.QCheckBox": {
        "addAction": True,
        "addActions": True,
        "animateClick": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hitButton": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoExclusive": True,
        "setAutoFillBackground": True,
        "setAutoRepeat": True,
        "setAutoRepeatDelay": True,
        "setAutoRepeatInterval": True,
        "setBackgroundRole": True,
        "setCheckState": True,
        "setCheckable": True,
        "setChecked": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setDown": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcut": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTristate": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QColorDialog": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCurrentColor": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOption": True,
        "setOptions": True,
        "setPalette": True,
        "setProperty": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "testOption": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QColumnView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "closePersistentEditor": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "createColumn": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "editorDestroyed": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "initializeColumn": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isPersistentEditorOpen": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "openPersistentEditor": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setColumnWidths": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPreviewWidget": True,
        "setProperty": True,
        "setResizeGripsVisible": True,
        "setRootIndex": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setupViewport": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualRect": True,
        "visualRegionForSelection": True,
        "wheelEvent": True
    },
    "QtWidgets.QComboBox": {
        "addAction": True,
        "addActions": True,
        "addItems": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "currentData": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "findData": True,
        "findText": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodEvent": True,
        "insertAction": True,
        "insertActions": True,
        "insertItems": True,
        "insertSeparator": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemData": True,
        "itemIcon": True,
        "itemText": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeItem": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCompleter": True,
        "setContextMenuPolicy": True,
        "setCurrentIndex": True,
        "setCurrentText": True,
        "setCursor": True,
        "setDisabled": True,
        "setDuplicatesEnabled": True,
        "setEditText": True,
        "setEditable": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setInsertPolicy": True,
        "setItemData": True,
        "setItemDelegate": True,
        "setItemIcon": True,
        "setItemText": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaxCount": True,
        "setMaxVisibleItems": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumContentsLength": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setModelColumn": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPlaceholderText": True,
        "setProperty": True,
        "setRootModelIndex": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setValidator": True,
        "setView": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QCommandLinkButton": {
        "addAction": True,
        "addActions": True,
        "animateClick": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusInEvent": True,
        "focusNextPrevChild": True,
        "focusOutEvent": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hitButton": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoDefault": True,
        "setAutoExclusive": True,
        "setAutoFillBackground": True,
        "setAutoRepeat": True,
        "setAutoRepeatDelay": True,
        "setAutoRepeatInterval": True,
        "setBackgroundRole": True,
        "setCheckable": True,
        "setChecked": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDefault": True,
        "setDescription": True,
        "setDisabled": True,
        "setDown": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFlat": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMenu": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcut": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QCommonStyle": {
        "blockSignals": True,
        "combinedLayoutSpacing": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "drawComplexControl": True,
        "drawControl": True,
        "drawItemPixmap": True,
        "drawItemText": True,
        "drawPrimitive": True,
        "findChild": True,
        "generatedIconPixmap": True,
        "hitTestComplexControl": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemPixmapRect": True,
        "itemTextRect": True,
        "killTimer": True,
        "layoutSpacing": True,
        "moveToThread": True,
        "pixelMetric": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sizeFromContents": True,
        "standardIcon": True,
        "standardPixmap": True,
        "startTimer": True,
        "styleHint": True,
        "subControlRect": True,
        "subElementRect": True,
        "tr": True
    },
    "QtWidgets.QCompleter": {
        "blockSignals": True,
        "complete": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "pathFromIndex": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCaseSensitivity": True,
        "setCompletionColumn": True,
        "setCompletionMode": True,
        "setCompletionPrefix": True,
        "setCompletionRole": True,
        "setCurrentRow": True,
        "setFilterMode": True,
        "setMaxVisibleItems": True,
        "setModel": True,
        "setModelSorting": True,
        "setObjectName": True,
        "setParent": True,
        "setPopup": True,
        "setProperty": True,
        "setWidget": True,
        "setWrapAround": True,
        "splitPath": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QDataWidgetMapper": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "mappedPropertyName": True,
        "mappedSection": True,
        "mappedWidgetAt": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeMapping": True,
        "setCurrentIndex": True,
        "setCurrentModelIndex": True,
        "setItemDelegate": True,
        "setModel": True,
        "setObjectName": True,
        "setOrientation": True,
        "setParent": True,
        "setProperty": True,
        "setRootIndex": True,
        "setSubmitPolicy": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QDateEdit": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "dateTimeFromText": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "sectionAt": True,
        "sectionText": True,
        "setAccelerated": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButtonSymbols": True,
        "setCalendar": True,
        "setCalendarPopup": True,
        "setCalendarWidget": True,
        "setContextMenuPolicy": True,
        "setCorrectionMode": True,
        "setCurrentSection": True,
        "setCurrentSectionIndex": True,
        "setCursor": True,
        "setDate": True,
        "setDateRange": True,
        "setDateTime": True,
        "setDateTimeRange": True,
        "setDisabled": True,
        "setDisplayFormat": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setGroupSeparatorShown": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeyboardTracking": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaximumDate": True,
        "setMaximumDateTime": True,
        "setMaximumHeight": True,
        "setMaximumTime": True,
        "setMaximumWidth": True,
        "setMinimumDate": True,
        "setMinimumDateTime": True,
        "setMinimumHeight": True,
        "setMinimumTime": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setReadOnly": True,
        "setSelectedSection": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSpecialValueText": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTime": True,
        "setTimeRange": True,
        "setTimeSpec": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWrapping": True,
        "stackUnder": True,
        "startTimer": True,
        "stepBy": True,
        "testAttribute": True,
        "textFromDateTime": True,
        "tr": True,
        "ungrabGesture": True,
        "validate": True
    },
    "QtWidgets.QDateTimeEdit": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "dateTimeFromText": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "sectionAt": True,
        "sectionText": True,
        "setAccelerated": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButtonSymbols": True,
        "setCalendar": True,
        "setCalendarPopup": True,
        "setCalendarWidget": True,
        "setContextMenuPolicy": True,
        "setCorrectionMode": True,
        "setCurrentSection": True,
        "setCurrentSectionIndex": True,
        "setCursor": True,
        "setDate": True,
        "setDateRange": True,
        "setDateTime": True,
        "setDateTimeRange": True,
        "setDisabled": True,
        "setDisplayFormat": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setGroupSeparatorShown": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeyboardTracking": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaximumDate": True,
        "setMaximumDateTime": True,
        "setMaximumHeight": True,
        "setMaximumTime": True,
        "setMaximumWidth": True,
        "setMinimumDate": True,
        "setMinimumDateTime": True,
        "setMinimumHeight": True,
        "setMinimumTime": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setReadOnly": True,
        "setSelectedSection": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSpecialValueText": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTime": True,
        "setTimeRange": True,
        "setTimeSpec": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWrapping": True,
        "stackUnder": True,
        "startTimer": True,
        "stepBy": True,
        "testAttribute": True,
        "textFromDateTime": True,
        "tr": True,
        "ungrabGesture": True,
        "validate": True
    },
    "QtWidgets.QDesktopWidget": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QDial": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setInvertedAppearance": True,
        "setInvertedControls": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setNotchTarget": True,
        "setNotchesVisible": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPageStep": True,
        "setPalette": True,
        "setProperty": True,
        "setRange": True,
        "setRepeatAction": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSingleStep": True,
        "setSliderDown": True,
        "setSliderPosition": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTracking": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWrapping": True,
        "sliderChange": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "triggerAction": True,
        "ungrabGesture": True
    },
    "QtWidgets.QDialog": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QDialogButtonBox": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "button": True,
        "buttonRole": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeButton": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCenterButtons": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStandardButtons": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "standardButton": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QDirModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "fileIcon": True,
        "fileInfo": True,
        "fileName": True,
        "filePath": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isDir": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "match": True,
        "mimeData": True,
        "mkdir": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "refresh": True,
        "remove": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rmdir": True,
        "rowCount": True,
        "setData": True,
        "setFilter": True,
        "setHeaderData": True,
        "setIconProvider": True,
        "setItemData": True,
        "setLazyChildCount": True,
        "setNameFilters": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setReadOnly": True,
        "setResolveSymlinks": True,
        "setSorting": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QDockWidget": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isAreaAllowed": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAllowedAreas": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFeatures": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFloating": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTitleBarWidget": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWidget": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QDoubleSpinBox": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAccelerated": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButtonSymbols": True,
        "setContextMenuPolicy": True,
        "setCorrectionMode": True,
        "setCursor": True,
        "setDecimals": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setGroupSeparatorShown": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeyboardTracking": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPrefix": True,
        "setProperty": True,
        "setRange": True,
        "setReadOnly": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSingleStep": True,
        "setSpecialValueText": True,
        "setStatusTip": True,
        "setStepType": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setSuffix": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWrapping": True,
        "stackUnder": True,
        "startTimer": True,
        "stepBy": True,
        "testAttribute": True,
        "textFromValue": True,
        "tr": True,
        "ungrabGesture": True,
        "validate": True,
        "valueFromText": True
    },
    "QtWidgets.QErrorMessage": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QFileDialog": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "labelText": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "restoreState": True,
        "selectFile": True,
        "selectMimeTypeFilter": True,
        "selectNameFilter": True,
        "selectUrl": True,
        "setAcceptDrops": True,
        "setAcceptMode": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDefaultSuffix": True,
        "setDirectoryUrl": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFileMode": True,
        "setFilter": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHistory": True,
        "setIconProvider": True,
        "setInputMethodHints": True,
        "setItemDelegate": True,
        "setLabelText": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMimeTypeFilters": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setNameFilter": True,
        "setNameFilters": True,
        "setObjectName": True,
        "setOption": True,
        "setOptions": True,
        "setPalette": True,
        "setProperty": True,
        "setProxyModel": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSidebarUrls": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setSupportedSchemes": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setViewMode": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "testOption": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QFileIconProvider": {
        "setOptions": True,
        "type": True
    },
    "QtWidgets.QFileSystemModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "fileIcon": True,
        "fileInfo": True,
        "fileName": True,
        "filePath": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isDir": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "lastModified": True,
        "match": True,
        "mimeData": True,
        "mkdir": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "myComputer": True,
        "property": True,
        "receivers": True,
        "remove": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rmdir": True,
        "rowCount": True,
        "setData": True,
        "setFilter": True,
        "setHeaderData": True,
        "setIconProvider": True,
        "setItemData": True,
        "setNameFilterDisables": True,
        "setNameFilters": True,
        "setObjectName": True,
        "setOption": True,
        "setOptions": True,
        "setParent": True,
        "setProperty": True,
        "setReadOnly": True,
        "setResolveSymlinks": True,
        "setRootPath": True,
        "sibling": True,
        "size": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "testOption": True,
        "tr": True,
        "type": True
    },
    "QtWidgets.QFocusFrame": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWidget": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QFontComboBox": {
        "addAction": True,
        "addActions": True,
        "addItems": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "currentData": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "findData": True,
        "findText": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodEvent": True,
        "insertAction": True,
        "insertActions": True,
        "insertItems": True,
        "insertSeparator": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemData": True,
        "itemIcon": True,
        "itemText": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeItem": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCompleter": True,
        "setContextMenuPolicy": True,
        "setCurrentFont": True,
        "setCurrentIndex": True,
        "setCurrentText": True,
        "setCursor": True,
        "setDisabled": True,
        "setDuplicatesEnabled": True,
        "setEditText": True,
        "setEditable": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setFontFilters": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setInsertPolicy": True,
        "setItemData": True,
        "setItemDelegate": True,
        "setItemIcon": True,
        "setItemText": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaxCount": True,
        "setMaxVisibleItems": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumContentsLength": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setModelColumn": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPlaceholderText": True,
        "setProperty": True,
        "setRootModelIndex": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setValidator": True,
        "setView": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWritingSystem": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QFontDialog": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCurrentFont": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOption": True,
        "setOptions": True,
        "setPalette": True,
        "setProperty": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "testOption": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QFormLayout": {
        "addChildLayout": True,
        "addChildWidget": True,
        "addItem": True,
        "addWidget": True,
        "alignmentRect": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "getItemPosition": True,
        "getLayoutPosition": True,
        "getWidgetPosition": True,
        "heightForWidth": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "minimumHeightForWidth": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "removeWidget": True,
        "replaceWidget": True,
        "setEnabled": True,
        "setFieldGrowthPolicy": True,
        "setFormAlignment": True,
        "setGeometry": True,
        "setHorizontalSpacing": True,
        "setItem": True,
        "setLabelAlignment": True,
        "setLayout": True,
        "setMenuBar": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRowWrapPolicy": True,
        "setSizeConstraint": True,
        "setSpacing": True,
        "setVerticalSpacing": True,
        "setWidget": True,
        "startTimer": True,
        "takeAt": True,
        "totalHeightForWidth": True,
        "tr": True,
        "widgetEvent": True
    },
    "QtWidgets.QFrame": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QGesture": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setGestureCancelPolicy": True,
        "setHotSpot": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGestureEvent": {
        "gesture": True,
        "mapToGraphicsScene": True
    },
    "QtWidgets.QGestureRecognizer": {
        "create": True,
        "reset": True
    },
    "QtWidgets.QGraphicsAnchor": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSizePolicy": True,
        "setSpacing": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsAnchorLayout": {
        "addAnchor": True,
        "addAnchors": True,
        "addChildLayoutItem": True,
        "addCornerAnchors": True,
        "anchor": True,
        "effectiveSizeHint": True,
        "itemAt": True,
        "removeAt": True,
        "setContentsMargins": True,
        "setGeometry": True,
        "setGraphicsItem": True,
        "setHorizontalSpacing": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setOwnedByLayout": True,
        "setParentLayoutItem": True,
        "setPreferredHeight": True,
        "setPreferredWidth": True,
        "setSpacing": True,
        "setVerticalSpacing": True,
        "sizeHint": True
    },
    "QtWidgets.QGraphicsBlurEffect": {
        "blockSignals": True,
        "boundingRectFor": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "draw": True,
        "drawSource": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setBlurHints": True,
        "setBlurRadius": True,
        "setEnabled": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sourceBoundingRect": True,
        "sourceChanged": True,
        "sourcePixmap": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsColorizeEffect": {
        "blockSignals": True,
        "boundingRectFor": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "draw": True,
        "drawSource": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setColor": True,
        "setEnabled": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setStrength": True,
        "sourceBoundingRect": True,
        "sourceChanged": True,
        "sourcePixmap": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsDropShadowEffect": {
        "blockSignals": True,
        "boundingRectFor": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "draw": True,
        "drawSource": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setBlurRadius": True,
        "setColor": True,
        "setEnabled": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setXOffset": True,
        "setYOffset": True,
        "sourceBoundingRect": True,
        "sourceChanged": True,
        "sourcePixmap": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsEffect": {
        "blockSignals": True,
        "boundingRectFor": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "draw": True,
        "drawSource": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setEnabled": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sourceBoundingRect": True,
        "sourceChanged": True,
        "sourcePixmap": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsEllipseItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setBrush": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPen": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setSpanAngle": True,
        "setStartAngle": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsGridLayout": {
        "addChildLayoutItem": True,
        "alignment": True,
        "columnAlignment": True,
        "columnMaximumWidth": True,
        "columnMinimumWidth": True,
        "columnPreferredWidth": True,
        "columnSpacing": True,
        "columnStretchFactor": True,
        "effectiveSizeHint": True,
        "removeAt": True,
        "removeItem": True,
        "rowAlignment": True,
        "rowMaximumHeight": True,
        "rowMinimumHeight": True,
        "rowPreferredHeight": True,
        "rowSpacing": True,
        "rowStretchFactor": True,
        "setAlignment": True,
        "setColumnAlignment": True,
        "setColumnFixedWidth": True,
        "setColumnMaximumWidth": True,
        "setColumnMinimumWidth": True,
        "setColumnPreferredWidth": True,
        "setColumnSpacing": True,
        "setColumnStretchFactor": True,
        "setContentsMargins": True,
        "setGeometry": True,
        "setGraphicsItem": True,
        "setHorizontalSpacing": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setOwnedByLayout": True,
        "setParentLayoutItem": True,
        "setPreferredHeight": True,
        "setPreferredWidth": True,
        "setRowAlignment": True,
        "setRowFixedHeight": True,
        "setRowMaximumHeight": True,
        "setRowMinimumHeight": True,
        "setRowPreferredHeight": True,
        "setRowSpacing": True,
        "setRowStretchFactor": True,
        "setSpacing": True,
        "setVerticalSpacing": True,
        "sizeHint": True
    },
    "QtWidgets.QGraphicsItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsItemGroup": {
        "addToGroup": True,
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeFromGroup": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsLayout": {
        "addChildLayoutItem": True,
        "effectiveSizeHint": True,
        "itemAt": True,
        "removeAt": True,
        "setContentsMargins": True,
        "setGeometry": True,
        "setGraphicsItem": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setOwnedByLayout": True,
        "setParentLayoutItem": True,
        "setPreferredHeight": True,
        "setPreferredWidth": True,
        "sizeHint": True
    },
    "QtWidgets.QGraphicsLayoutItem": {
        "effectiveSizeHint": True,
        "setGeometry": True,
        "setGraphicsItem": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setOwnedByLayout": True,
        "setParentLayoutItem": True,
        "setPreferredHeight": True,
        "setPreferredWidth": True,
        "sizeHint": True
    },
    "QtWidgets.QGraphicsLineItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPen": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsLinearLayout": {
        "addChildLayoutItem": True,
        "addItem": True,
        "addStretch": True,
        "alignment": True,
        "effectiveSizeHint": True,
        "insertItem": True,
        "insertStretch": True,
        "itemAt": True,
        "itemSpacing": True,
        "removeAt": True,
        "removeItem": True,
        "setAlignment": True,
        "setContentsMargins": True,
        "setGeometry": True,
        "setGraphicsItem": True,
        "setItemSpacing": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setOrientation": True,
        "setOwnedByLayout": True,
        "setParentLayoutItem": True,
        "setPreferredHeight": True,
        "setPreferredWidth": True,
        "setSpacing": True,
        "setStretchFactor": True,
        "sizeHint": True,
        "stretchFactor": True
    },
    "QtWidgets.QGraphicsObject": {
        "advance": True,
        "blockSignals": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "connectNotify": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "grabGesture": True,
        "inherits": True,
        "inputMethodQuery": True,
        "installEventFilter": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "killTimer": True,
        "moveBy": True,
        "moveToThread": True,
        "paint": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setObjectName": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParent": True,
        "setParentItem": True,
        "setProperty": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True,
        "startTimer": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QGraphicsOpacityEffect": {
        "blockSignals": True,
        "boundingRectFor": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "draw": True,
        "drawSource": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setEnabled": True,
        "setObjectName": True,
        "setOpacity": True,
        "setOpacityMask": True,
        "setParent": True,
        "setProperty": True,
        "sourceBoundingRect": True,
        "sourceChanged": True,
        "sourcePixmap": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsPathItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setBrush": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPath": True,
        "setPen": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsPixmapItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPixmap": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setShapeMode": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformationMode": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsPolygonItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setBrush": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFillRule": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPen": True,
        "setPolygon": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsProxyWidget": {
        "addAction": True,
        "addActions": True,
        "advance": True,
        "blockSignals": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "connectNotify": True,
        "contains": True,
        "createProxyForChildWidget": True,
        "data": True,
        "deviceTransform": True,
        "disconnectNotify": True,
        "effectiveSizeHint": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grabGesture": True,
        "grabShortcut": True,
        "inherits": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "killTimer": True,
        "moveBy": True,
        "moveToThread": True,
        "newProxyWidget": True,
        "paint": True,
        "paintWindowFrame": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setGeometry": True,
        "setGraphicsEffect": True,
        "setGraphicsItem": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setObjectName": True,
        "setOpacity": True,
        "setOwnedByLayout": True,
        "setPalette": True,
        "setPanelModality": True,
        "setParent": True,
        "setParentItem": True,
        "setParentLayoutItem": True,
        "setPreferredHeight": True,
        "setPreferredWidth": True,
        "setProperty": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStyle": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setWidget": True,
        "setWindowFlags": True,
        "setWindowTitle": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "sizeHint": True,
        "stackBefore": True,
        "startTimer": True,
        "subWidgetRect": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "windowFrameSectionAt": True
    },
    "QtWidgets.QGraphicsRectItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setBrush": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPen": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsRotation": {
        "applyTo": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setAngle": True,
        "setObjectName": True,
        "setOrigin": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsScale": {
        "applyTo": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setOrigin": True,
        "setParent": True,
        "setProperty": True,
        "setXScale": True,
        "setYScale": True,
        "setZScale": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsScene": {
        "addItem": True,
        "addPath": True,
        "addPixmap": True,
        "addPolygon": True,
        "addSimpleText": True,
        "addText": True,
        "addWidget": True,
        "blockSignals": True,
        "collidingItems": True,
        "connectNotify": True,
        "createItemGroup": True,
        "destroyItemGroup": True,
        "disconnectNotify": True,
        "drawBackground": True,
        "drawForeground": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "inherits": True,
        "inputMethodQuery": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "render": True,
        "setActivePanel": True,
        "setActiveWindow": True,
        "setBackgroundBrush": True,
        "setBspTreeDepth": True,
        "setFocus": True,
        "setFocusItem": True,
        "setFocusOnTouch": True,
        "setFont": True,
        "setForegroundBrush": True,
        "setItemIndexMethod": True,
        "setMinimumRenderSize": True,
        "setObjectName": True,
        "setPalette": True,
        "setParent": True,
        "setProperty": True,
        "setStickyFocus": True,
        "setStyle": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsSceneContextMenuEvent": {
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSceneDragDropEvent": {
        "setAccepted": True,
        "setDropAction": True
    },
    "QtWidgets.QGraphicsSceneEvent": {
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSceneHelpEvent": {
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSceneHoverEvent": {
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSceneMouseEvent": {
        "buttonDownPos": True,
        "buttonDownScenePos": True,
        "buttonDownScreenPos": True,
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSceneMoveEvent": {
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSceneResizeEvent": {
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSceneWheelEvent": {
        "setAccepted": True
    },
    "QtWidgets.QGraphicsSimpleTextItem": {
        "advance": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "inputMethodQuery": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "moveBy": True,
        "paint": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setBrush": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setFont": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setOpacity": True,
        "setPanelModality": True,
        "setParentItem": True,
        "setPen": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setText": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True
    },
    "QtWidgets.QGraphicsTextItem": {
        "advance": True,
        "blockSignals": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "connectNotify": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "grabGesture": True,
        "inherits": True,
        "inputMethodQuery": True,
        "installEventFilter": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "killTimer": True,
        "moveBy": True,
        "moveToThread": True,
        "paint": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setDefaultTextColor": True,
        "setDocument": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusProxy": True,
        "setFont": True,
        "setGraphicsEffect": True,
        "setGroup": True,
        "setHtml": True,
        "setInputMethodHints": True,
        "setObjectName": True,
        "setOpacity": True,
        "setOpenExternalLinks": True,
        "setPanelModality": True,
        "setParent": True,
        "setParentItem": True,
        "setPlainText": True,
        "setProperty": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setTabChangesFocus": True,
        "setTextCursor": True,
        "setTextInteractionFlags": True,
        "setTextWidth": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "stackBefore": True,
        "startTimer": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QGraphicsTransform": {
        "applyTo": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QGraphicsView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawBackground": True,
        "drawForeground": True,
        "drawFrame": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "invalidateScene": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "rotate": True,
        "scale": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundBrush": True,
        "setBackgroundRole": True,
        "setCacheMode": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCursor": True,
        "setDisabled": True,
        "setDragMode": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundBrush": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setInputMethodHints": True,
        "setInteractive": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOptimizationFlag": True,
        "setOptimizationFlags": True,
        "setPalette": True,
        "setProperty": True,
        "setRenderHint": True,
        "setRenderHints": True,
        "setResizeAnchor": True,
        "setRubberBandSelectionMode": True,
        "setScene": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTransform": True,
        "setTransformationAnchor": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setViewport": True,
        "setViewportUpdateMode": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setupViewport": True,
        "shear": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "translate": True,
        "ungrabGesture": True,
        "updateScene": True,
        "updateSceneRect": True
    },
    "QtWidgets.QGraphicsWidget": {
        "addAction": True,
        "addActions": True,
        "advance": True,
        "blockSignals": True,
        "boundingRegion": True,
        "collidesWithItem": True,
        "collidesWithPath": True,
        "collidingItems": True,
        "commonAncestorItem": True,
        "connectNotify": True,
        "contains": True,
        "data": True,
        "deviceTransform": True,
        "disconnectNotify": True,
        "effectiveSizeHint": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grabGesture": True,
        "grabShortcut": True,
        "inherits": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "installSceneEventFilter": True,
        "isAncestorOf": True,
        "isBlockedByModalPanel": True,
        "isObscuredBy": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemChange": True,
        "itemTransform": True,
        "killTimer": True,
        "moveBy": True,
        "moveToThread": True,
        "paint": True,
        "paintWindowFrame": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeSceneEventFilter": True,
        "scroll": True,
        "setAcceptDrops": True,
        "setAcceptHoverEvents": True,
        "setAcceptTouchEvents": True,
        "setAcceptedMouseButtons": True,
        "setActive": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBoundingRegionGranularity": True,
        "setCacheMode": True,
        "setCursor": True,
        "setData": True,
        "setEnabled": True,
        "setFiltersChildEvents": True,
        "setFlag": True,
        "setFlags": True,
        "setFocus": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setGraphicsEffect": True,
        "setGraphicsItem": True,
        "setGroup": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setObjectName": True,
        "setOpacity": True,
        "setOwnedByLayout": True,
        "setPalette": True,
        "setPanelModality": True,
        "setParent": True,
        "setParentItem": True,
        "setParentLayoutItem": True,
        "setPreferredHeight": True,
        "setPreferredWidth": True,
        "setProperty": True,
        "setRotation": True,
        "setScale": True,
        "setSelected": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStyle": True,
        "setToolTip": True,
        "setTransform": True,
        "setTransformations": True,
        "setVisible": True,
        "setWindowFlags": True,
        "setWindowTitle": True,
        "setX": True,
        "setY": True,
        "setZValue": True,
        "sizeHint": True,
        "stackBefore": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "windowFrameSectionAt": True
    },
    "QtWidgets.QGridLayout": {
        "addChildLayout": True,
        "addChildWidget": True,
        "alignmentRect": True,
        "blockSignals": True,
        "cellRect": True,
        "columnMinimumWidth": True,
        "columnStretch": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "getItemPosition": True,
        "heightForWidth": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemAt": True,
        "itemAtPosition": True,
        "killTimer": True,
        "minimumHeightForWidth": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "removeWidget": True,
        "replaceWidget": True,
        "rowMinimumHeight": True,
        "rowStretch": True,
        "setColumnMinimumWidth": True,
        "setColumnStretch": True,
        "setDefaultPositioning": True,
        "setEnabled": True,
        "setGeometry": True,
        "setHorizontalSpacing": True,
        "setMenuBar": True,
        "setObjectName": True,
        "setOriginCorner": True,
        "setParent": True,
        "setProperty": True,
        "setRowMinimumHeight": True,
        "setRowStretch": True,
        "setSizeConstraint": True,
        "setSpacing": True,
        "setVerticalSpacing": True,
        "startTimer": True,
        "takeAt": True,
        "totalHeightForWidth": True,
        "tr": True,
        "widgetEvent": True
    },
    "QtWidgets.QGroupBox": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCheckable": True,
        "setChecked": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFlat": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTitle": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QHBoxLayout": {
        "addChildLayout": True,
        "addChildWidget": True,
        "addItem": True,
        "addLayout": True,
        "addSpacerItem": True,
        "addSpacing": True,
        "addStretch": True,
        "addStrut": True,
        "alignmentRect": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "heightForWidth": True,
        "inherits": True,
        "insertItem": True,
        "insertLayout": True,
        "insertSpacerItem": True,
        "insertSpacing": True,
        "insertStretch": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemAt": True,
        "killTimer": True,
        "minimumHeightForWidth": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "removeWidget": True,
        "replaceWidget": True,
        "setDirection": True,
        "setEnabled": True,
        "setGeometry": True,
        "setMenuBar": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSizeConstraint": True,
        "setSpacing": True,
        "setStretch": True,
        "startTimer": True,
        "stretch": True,
        "takeAt": True,
        "totalHeightForWidth": True,
        "tr": True,
        "widgetEvent": True
    },
    "QtWidgets.QHeaderView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "closePersistentEditor": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "editorDestroyed": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "headerDataChanged": True,
        "heightForWidth": True,
        "hideSection": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isPersistentEditorOpen": True,
        "isSectionHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "keyboardSearch": True,
        "killTimer": True,
        "logicalIndex": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveCursor": True,
        "moveSection": True,
        "moveToThread": True,
        "nativeEvent": True,
        "openPersistentEditor": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintSection": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeSection": True,
        "restoreGeometry": True,
        "restoreState": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "sectionPosition": True,
        "sectionResizeMode": True,
        "sectionSize": True,
        "sectionSizeFromContents": True,
        "sectionSizeHint": True,
        "sectionViewportPosition": True,
        "sectionsAboutToBeRemoved": True,
        "sectionsInserted": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setCascadingSectionResizes": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultAlignment": True,
        "setDefaultDropAction": True,
        "setDefaultSectionSize": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setFirstSectionMovable": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHighlightSections": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumSectionSize": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumSectionSize": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOffset": True,
        "setOffsetToSectionPosition": True,
        "setPalette": True,
        "setProperty": True,
        "setResizeContentsPrecision": True,
        "setRootIndex": True,
        "setSectionHidden": True,
        "setSectionsClickable": True,
        "setSectionsMovable": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setSortIndicator": True,
        "setSortIndicatorShown": True,
        "setState": True,
        "setStatusTip": True,
        "setStretchLastSection": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setupViewport": True,
        "showSection": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "swapSections": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "updateSection": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualIndex": True,
        "visualIndexAt": True,
        "visualRect": True,
        "visualRegionForSelection": True,
        "wheelEvent": True
    },
    "QtWidgets.QInputDialog": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCancelButtonText": True,
        "setComboBoxEditable": True,
        "setComboBoxItems": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setDoubleDecimals": True,
        "setDoubleMaximum": True,
        "setDoubleMinimum": True,
        "setDoubleRange": True,
        "setDoubleStep": True,
        "setDoubleValue": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setInputMode": True,
        "setIntMaximum": True,
        "setIntMinimum": True,
        "setIntRange": True,
        "setIntStep": True,
        "setIntValue": True,
        "setLabelText": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOkButtonText": True,
        "setOption": True,
        "setPalette": True,
        "setProperty": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTextEchoMode": True,
        "setTextValue": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "testOption": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QItemDelegate": {
        "blockSignals": True,
        "connectNotify": True,
        "createEditor": True,
        "destroyEditor": True,
        "disconnectNotify": True,
        "drawBackground": True,
        "drawCheck": True,
        "drawDecoration": True,
        "drawDisplay": True,
        "drawFocus": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "paint": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setClipping": True,
        "setEditorData": True,
        "setItemEditorFactory": True,
        "setModelData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sizeHint": True,
        "startTimer": True,
        "tr": True,
        "updateEditorGeometry": True
    },
    "QtWidgets.QItemEditorCreatorBase": {
        "createWidget": True
    },
    "QtWidgets.QItemEditorFactory": {
        "createEditor": True,
        "registerEditor": True,
        "valuePropertyName": True
    },
    "QtWidgets.QKeyEventTransition": {
        "addAnimation": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setEventSource": True,
        "setEventType": True,
        "setKey": True,
        "setModifierMask": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setTargetState": True,
        "setTargetStates": True,
        "setTransitionType": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QKeySequenceEdit": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "keyReleaseEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeySequence": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QLCDNumber": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDigitCount": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMode": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setSegmentStyle": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSmallDecimalPoint": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QLabel": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "findChild": True,
        "focusInEvent": True,
        "focusNextPrevChild": True,
        "focusOutEvent": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setBuddy": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIndent": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMargin": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setMovie": True,
        "setObjectName": True,
        "setOpenExternalLinks": True,
        "setPalette": True,
        "setPicture": True,
        "setPixmap": True,
        "setProperty": True,
        "setScaledContents": True,
        "setSelection": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setTextFormat": True,
        "setTextInteractionFlags": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QLayout": {
        "addChildLayout": True,
        "addChildWidget": True,
        "addItem": True,
        "addWidget": True,
        "alignmentRect": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "heightForWidth": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemAt": True,
        "killTimer": True,
        "minimumHeightForWidth": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "removeWidget": True,
        "replaceWidget": True,
        "setEnabled": True,
        "setGeometry": True,
        "setMenuBar": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSizeConstraint": True,
        "setSpacing": True,
        "startTimer": True,
        "takeAt": True,
        "totalHeightForWidth": True,
        "tr": True,
        "widgetEvent": True
    },
    "QtWidgets.QLayoutItem": {
        "heightForWidth": True,
        "minimumHeightForWidth": True,
        "setAlignment": True,
        "setGeometry": True
    },
    "QtWidgets.QLineEdit": {
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "cursorBackward": True,
        "cursorForward": True,
        "cursorPositionAt": True,
        "cursorWordBackward": True,
        "cursorWordForward": True,
        "destroy": True,
        "disconnectNotify": True,
        "dragEnterEvent": True,
        "dropEvent": True,
        "end": True,
        "event": True,
        "findChild": True,
        "focusInEvent": True,
        "focusNextPrevChild": True,
        "focusOutEvent": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "home": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodEvent": True,
        "insert": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setClearButtonEnabled": True,
        "setCompleter": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setCursorMoveStyle": True,
        "setCursorPosition": True,
        "setDisabled": True,
        "setDragEnabled": True,
        "setEchoMode": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMask": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaxLength": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModified": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPlaceholderText": True,
        "setProperty": True,
        "setReadOnly": True,
        "setSelection": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setValidator": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QListView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "closePersistentEditor": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "editorDestroyed": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isPersistentEditorOpen": True,
        "isRowHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "openPersistentEditor": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "rectForIndex": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setBatchSize": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFlow": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setGridSize": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemAlignment": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLayoutMode": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setModelColumn": True,
        "setMouseTracking": True,
        "setMovement": True,
        "setObjectName": True,
        "setPalette": True,
        "setPositionForIndex": True,
        "setProperty": True,
        "setResizeMode": True,
        "setRootIndex": True,
        "setRowHidden": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setSelectionRectVisible": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setSpacing": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUniformItemSizes": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "setWrapping": True,
        "setupViewport": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualRect": True,
        "visualRegionForSelection": True
    },
    "QtWidgets.QListWidget": {
        "addAction": True,
        "addActions": True,
        "addItems": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "dropMimeData": True,
        "editItem": True,
        "editorDestroyed": True,
        "findChild": True,
        "findItems": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexFromItem": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertItems": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isRowHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "item": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "itemFromIndex": True,
        "itemWidget": True,
        "items": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mimeData": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "rectForIndex": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeItemWidget": True,
        "restoreGeometry": True,
        "row": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "scrollToItem": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setBatchSize": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFlow": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setGridSize": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemAlignment": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setItemWidget": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLayoutMode": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setModelColumn": True,
        "setMouseTracking": True,
        "setMovement": True,
        "setObjectName": True,
        "setPalette": True,
        "setPositionForIndex": True,
        "setProperty": True,
        "setResizeMode": True,
        "setRootIndex": True,
        "setRowHidden": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setSelectionRectVisible": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setSortingEnabled": True,
        "setSpacing": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUniformItemSizes": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "setWrapping": True,
        "setupViewport": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "sortItems": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "takeItem": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualItemRect": True,
        "visualRect": True,
        "visualRegionForSelection": True
    },
    "QtWidgets.QListWidgetItem": {
        "data": True,
        "read": True,
        "setBackground": True,
        "setCheckState": True,
        "setData": True,
        "setFlags": True,
        "setFont": True,
        "setForeground": True,
        "setHidden": True,
        "setIcon": True,
        "setSelected": True,
        "setSizeHint": True,
        "setStatusTip": True,
        "setText": True,
        "setTextAlignment": True,
        "setToolTip": True,
        "setWhatsThis": True,
        "write": True
    },
    "QtWidgets.QMainWindow": {
        "addAction": True,
        "addActions": True,
        "addToolBarBreak": True,
        "blockSignals": True,
        "connectNotify": True,
        "corner": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "dockWidgetArea": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertToolBar": True,
        "insertToolBarBreak": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSeparator": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeDockWidget": True,
        "removeEventFilter": True,
        "removeToolBar": True,
        "removeToolBarBreak": True,
        "resizeDocks": True,
        "restoreDockWidget": True,
        "restoreGeometry": True,
        "restoreState": True,
        "saveState": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAnimated": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCentralWidget": True,
        "setContextMenuPolicy": True,
        "setCorner": True,
        "setCursor": True,
        "setDisabled": True,
        "setDockNestingEnabled": True,
        "setDockOptions": True,
        "setDocumentMode": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMenuBar": True,
        "setMenuWidget": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusBar": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabPosition": True,
        "setTabShape": True,
        "setTabletTracking": True,
        "setToolButtonStyle": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUnifiedTitleAndToolBarOnMac": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "splitDockWidget": True,
        "stackUnder": True,
        "startTimer": True,
        "tabPosition": True,
        "tabifiedDockWidgets": True,
        "tabifyDockWidget": True,
        "testAttribute": True,
        "toolBarArea": True,
        "toolBarBreak": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QMdiArea": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "addSubWindow": True,
        "blockSignals": True,
        "changeEvent": True,
        "childEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "dragEnterEvent": True,
        "dragLeaveEvent": True,
        "dragMoveEvent": True,
        "drawFrame": True,
        "dropEvent": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeSubWindow": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setActivationOrder": True,
        "setActiveSubWindow": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCursor": True,
        "setDisabled": True,
        "setDocumentMode": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOption": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabPosition": True,
        "setTabShape": True,
        "setTabletTracking": True,
        "setTabsClosable": True,
        "setTabsMovable": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setViewMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setupViewport": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "subWindowList": True,
        "testAttribute": True,
        "testOption": True,
        "timerEvent": True,
        "tr": True,
        "ungrabGesture": True,
        "wheelEvent": True
    },
    "QtWidgets.QMdiSubWindow": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "childEvent": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusInEvent": True,
        "focusNextPrevChild": True,
        "focusOutEvent": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideEvent": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "leaveEvent": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeyboardPageStep": True,
        "setKeyboardSingleStep": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOption": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setSystemMenu": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWidget": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "testOption": True,
        "timerEvent": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QMenu": {
        "actionAt": True,
        "actionEvent": True,
        "actionGeometry": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "enterEvent": True,
        "event": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideEvent": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertMenu": True,
        "insertSeparator": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "leaveEvent": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "popup": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setActiveAction": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDefaultAction": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setSeparatorsCollapsible": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTearOffEnabled": True,
        "setTitle": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setToolTipsVisible": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "ungrabGesture": True,
        "wheelEvent": True
    },
    "QtWidgets.QMenuBar": {
        "actionAt": True,
        "actionEvent": True,
        "actionGeometry": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "cornerWidget": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "event": True,
        "eventFilter": True,
        "findChild": True,
        "focusInEvent": True,
        "focusNextPrevChild": True,
        "focusOutEvent": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertMenu": True,
        "insertSeparator": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "leaveEvent": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setActiveAction": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCursor": True,
        "setDefaultUp": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setNativeMenuBar": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QMessageBox": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "button": True,
        "buttonRole": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeButton": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setCheckBox": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDetailedText": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setIconPixmap": True,
        "setInformativeText": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStandardButtons": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setTextFormat": True,
        "setTextInteractionFlags": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "standardButton": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QMouseEventTransition": {
        "addAnimation": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setButton": True,
        "setEventSource": True,
        "setEventType": True,
        "setHitTestPath": True,
        "setModifierMask": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setTargetState": True,
        "setTargetStates": True,
        "setTransitionType": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QOpenGLWidget": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeGL": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFormat": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTextureFormat": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdateBehavior": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QPanGesture": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setAcceleration": True,
        "setGestureCancelPolicy": True,
        "setHotSpot": True,
        "setLastOffset": True,
        "setObjectName": True,
        "setOffset": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QPinchGesture": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCenterPoint": True,
        "setChangeFlags": True,
        "setGestureCancelPolicy": True,
        "setHotSpot": True,
        "setLastCenterPoint": True,
        "setLastRotationAngle": True,
        "setLastScaleFactor": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRotationAngle": True,
        "setScaleFactor": True,
        "setStartCenterPoint": True,
        "setTotalChangeFlags": True,
        "setTotalRotationAngle": True,
        "setTotalScaleFactor": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QPlainTextDocumentLayout": {
        "anchorAt": True,
        "blockBoundingRect": True,
        "blockSignals": True,
        "blockWithMarkerAt": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "documentChanged": True,
        "draw": True,
        "drawInlineObject": True,
        "ensureBlockLayout": True,
        "findChild": True,
        "format": True,
        "formatAt": True,
        "frameBoundingRect": True,
        "handlerForObject": True,
        "hitTest": True,
        "imageAt": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "positionInlineObject": True,
        "property": True,
        "receivers": True,
        "registerHandler": True,
        "removeEventFilter": True,
        "resizeInlineObject": True,
        "setCursorWidth": True,
        "setObjectName": True,
        "setPaintDevice": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True,
        "unregisterHandler": True
    },
    "QtWidgets.QPlainTextEdit": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "anchorAt": True,
        "appendHtml": True,
        "appendPlainText": True,
        "blockBoundingGeometry": True,
        "blockBoundingRect": True,
        "blockSignals": True,
        "canInsertFromMimeData": True,
        "connectNotify": True,
        "create": True,
        "cursorForPosition": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodEvent": True,
        "insertAction": True,
        "insertActions": True,
        "insertFromMimeData": True,
        "insertPlainText": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "loadResource": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "mergeCurrentCharFormat": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "print_": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setBackgroundVisible": True,
        "setCenterOnScroll": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentCharFormat": True,
        "setCursor": True,
        "setCursorWidth": True,
        "setDisabled": True,
        "setDocument": True,
        "setDocumentTitle": True,
        "setEnabled": True,
        "setExtraSelections": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLineWrapMode": True,
        "setLocale": True,
        "setMaximumBlockCount": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOverwriteMode": True,
        "setPalette": True,
        "setPlaceholderText": True,
        "setPlainText": True,
        "setProperty": True,
        "setReadOnly": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabChangesFocus": True,
        "setTabStopDistance": True,
        "setTabStopWidth": True,
        "setTabletTracking": True,
        "setTextCursor": True,
        "setTextInteractionFlags": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUndoRedoEnabled": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrapMode": True,
        "setupViewport": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "viewportEvent": True,
        "zoomIn": True,
        "zoomOut": True
    },
    "QtWidgets.QProgressBar": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFormat": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setInvertedAppearance": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPalette": True,
        "setProperty": True,
        "setRange": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTextDirection": True,
        "setTextVisible": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QProgressDialog": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoClose": True,
        "setAutoFillBackground": True,
        "setAutoReset": True,
        "setBackgroundRole": True,
        "setBar": True,
        "setCancelButton": True,
        "setCancelButtonText": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLabel": True,
        "setLabelText": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumDuration": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setRange": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QProxyStyle": {
        "blockSignals": True,
        "combinedLayoutSpacing": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "drawComplexControl": True,
        "drawControl": True,
        "drawItemPixmap": True,
        "drawItemText": True,
        "drawPrimitive": True,
        "findChild": True,
        "generatedIconPixmap": True,
        "hitTestComplexControl": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemPixmapRect": True,
        "itemTextRect": True,
        "killTimer": True,
        "layoutSpacing": True,
        "moveToThread": True,
        "pixelMetric": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setBaseStyle": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sizeFromContents": True,
        "standardIcon": True,
        "standardPixmap": True,
        "startTimer": True,
        "styleHint": True,
        "subControlRect": True,
        "subElementRect": True,
        "tr": True
    },
    "QtWidgets.QPushButton": {
        "addAction": True,
        "addActions": True,
        "animateClick": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusInEvent": True,
        "focusNextPrevChild": True,
        "focusOutEvent": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hitButton": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoDefault": True,
        "setAutoExclusive": True,
        "setAutoFillBackground": True,
        "setAutoRepeat": True,
        "setAutoRepeatDelay": True,
        "setAutoRepeatInterval": True,
        "setBackgroundRole": True,
        "setCheckable": True,
        "setChecked": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDefault": True,
        "setDisabled": True,
        "setDown": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFlat": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMenu": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcut": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QRadioButton": {
        "addAction": True,
        "addActions": True,
        "animateClick": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hitButton": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoExclusive": True,
        "setAutoFillBackground": True,
        "setAutoRepeat": True,
        "setAutoRepeatDelay": True,
        "setAutoRepeatInterval": True,
        "setBackgroundRole": True,
        "setCheckable": True,
        "setChecked": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setDown": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcut": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QRubberBand": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QScrollArea": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "dragEnterEvent": True,
        "dragLeaveEvent": True,
        "dragMoveEvent": True,
        "drawFrame": True,
        "dropEvent": True,
        "ensureVisible": True,
        "ensureWidgetVisible": True,
        "event": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWidget": True,
        "setWidgetResizable": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setupViewport": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "viewportEvent": True,
        "wheelEvent": True
    },
    "QtWidgets.QScrollBar": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideEvent": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setInvertedAppearance": True,
        "setInvertedControls": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPageStep": True,
        "setPalette": True,
        "setProperty": True,
        "setRange": True,
        "setRepeatAction": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSingleStep": True,
        "setSliderDown": True,
        "setSliderPosition": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTracking": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "sliderChange": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "triggerAction": True,
        "ungrabGesture": True,
        "wheelEvent": True
    },
    "QtWidgets.QScroller": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "handleInput": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setScrollerProperties": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QScrollerProperties": {
        "scrollMetric": True,
        "setScrollMetric": True
    },
    "QtWidgets.QShortcut": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setAutoRepeat": True,
        "setContext": True,
        "setEnabled": True,
        "setKey": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setWhatsThis": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QSizeGrip": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "event": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideEvent": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QSizePolicy": {
        "setControlType": True,
        "setHeightForWidth": True,
        "setHorizontalPolicy": True,
        "setHorizontalStretch": True,
        "setRetainSizeWhenHidden": True,
        "setVerticalPolicy": True,
        "setVerticalStretch": True,
        "setWidthForHeight": True
    },
    "QtWidgets.QSlider": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setInvertedAppearance": True,
        "setInvertedControls": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPageStep": True,
        "setPalette": True,
        "setProperty": True,
        "setRange": True,
        "setRepeatAction": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSingleStep": True,
        "setSliderDown": True,
        "setSliderPosition": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTickInterval": True,
        "setTickPosition": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTracking": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "sliderChange": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "triggerAction": True,
        "ungrabGesture": True
    },
    "QtWidgets.QSpacerItem": {
        "changeSize": True,
        "heightForWidth": True,
        "minimumHeightForWidth": True,
        "setAlignment": True,
        "setGeometry": True
    },
    "QtWidgets.QSpinBox": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAccelerated": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButtonSymbols": True,
        "setContextMenuPolicy": True,
        "setCorrectionMode": True,
        "setCursor": True,
        "setDisabled": True,
        "setDisplayIntegerBase": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setGroupSeparatorShown": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeyboardTracking": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaximum": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimum": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPrefix": True,
        "setProperty": True,
        "setRange": True,
        "setReadOnly": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSingleStep": True,
        "setSpecialValueText": True,
        "setStatusTip": True,
        "setStepType": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setSuffix": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setValue": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWrapping": True,
        "stackUnder": True,
        "startTimer": True,
        "stepBy": True,
        "testAttribute": True,
        "textFromValue": True,
        "tr": True,
        "ungrabGesture": True,
        "validate": True,
        "valueFromText": True
    },
    "QtWidgets.QSplashScreen": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawContents": True,
        "findChild": True,
        "finish": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mousePressEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPixmap": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showMessage": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QSplitter": {
        "addAction": True,
        "addActions": True,
        "addWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "childEvent": True,
        "closestLegalPosition": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "event": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "getRange": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "handle": True,
        "heightForWidth": True,
        "indexOf": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isCollapsible": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveSplitter": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "replaceWidget": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "restoreState": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setChildrenCollapsible": True,
        "setCollapsible": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHandleWidth": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOpaqueResize": True,
        "setOrientation": True,
        "setPalette": True,
        "setProperty": True,
        "setRubberBand": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizes": True,
        "setStatusTip": True,
        "setStretchFactor": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "widget": True
    },
    "QtWidgets.QSplitterHandle": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "closestLegalPosition": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveSplitter": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QStackedLayout": {
        "addChildLayout": True,
        "addChildWidget": True,
        "addItem": True,
        "addWidget": True,
        "alignmentRect": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "heightForWidth": True,
        "inherits": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemAt": True,
        "killTimer": True,
        "minimumHeightForWidth": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "removeWidget": True,
        "replaceWidget": True,
        "setCurrentIndex": True,
        "setCurrentWidget": True,
        "setEnabled": True,
        "setGeometry": True,
        "setMenuBar": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSizeConstraint": True,
        "setSpacing": True,
        "setStackingMode": True,
        "startTimer": True,
        "takeAt": True,
        "totalHeightForWidth": True,
        "tr": True,
        "widgetEvent": True
    },
    "QtWidgets.QStackedWidget": {
        "addAction": True,
        "addActions": True,
        "addWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "indexOf": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeWidget": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCurrentIndex": True,
        "setCurrentWidget": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "widget": True
    },
    "QtWidgets.QStatusBar": {
        "addAction": True,
        "addActions": True,
        "addPermanentWidget": True,
        "addWidget": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertPermanentWidget": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeWidget": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeGripEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "showMessage": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QStyle": {
        "blockSignals": True,
        "combinedLayoutSpacing": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "drawComplexControl": True,
        "drawControl": True,
        "drawItemPixmap": True,
        "drawItemText": True,
        "drawPrimitive": True,
        "findChild": True,
        "generatedIconPixmap": True,
        "hitTestComplexControl": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemPixmapRect": True,
        "itemTextRect": True,
        "killTimer": True,
        "layoutSpacing": True,
        "moveToThread": True,
        "pixelMetric": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sizeFromContents": True,
        "standardIcon": True,
        "standardPixmap": True,
        "startTimer": True,
        "styleHint": True,
        "subControlRect": True,
        "subElementRect": True,
        "tr": True
    },
    "QtWidgets.QStyleOption": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionButton": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionComboBox": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionComplex": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionDockWidget": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionFocusRect": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionFrame": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionGraphicsItem": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionGroupBox": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionHeader": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionMenuItem": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionProgressBar": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionRubberBand": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionSizeGrip": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionSlider": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionSpinBox": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionTab": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionTabBarBase": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionTabWidgetFrame": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionTitleBar": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionToolBar": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionToolBox": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionToolButton": {
        "initFrom": True
    },
    "QtWidgets.QStyleOptionViewItem": {
        "initFrom": True
    },
    "QtWidgets.QStylePainter": {
        "drawComplexControl": True,
        "drawControl": True,
        "drawItemPixmap": True,
        "drawItemText": True,
        "drawPath": True,
        "drawPixmapFragments": True,
        "drawPrimitive": True,
        "fillPath": True,
        "rotate": True,
        "scale": True,
        "setBackground": True,
        "setBackgroundMode": True,
        "setClipPath": True,
        "setClipRegion": True,
        "setClipping": True,
        "setCompositionMode": True,
        "setFont": True,
        "setLayoutDirection": True,
        "setOpacity": True,
        "setRenderHint": True,
        "setRenderHints": True,
        "setTransform": True,
        "setViewTransformEnabled": True,
        "setWorldMatrixEnabled": True,
        "setWorldTransform": True,
        "shear": True,
        "strokePath": True,
        "testRenderHint": True
    },
    "QtWidgets.QStyledItemDelegate": {
        "blockSignals": True,
        "connectNotify": True,
        "createEditor": True,
        "destroyEditor": True,
        "disconnectNotify": True,
        "displayText": True,
        "findChild": True,
        "inherits": True,
        "initStyleOption": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "paint": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setEditorData": True,
        "setItemEditorFactory": True,
        "setModelData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sizeHint": True,
        "startTimer": True,
        "tr": True,
        "updateEditorGeometry": True
    },
    "QtWidgets.QSwipeGesture": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setGestureCancelPolicy": True,
        "setHotSpot": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSwipeAngle": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QSystemTrayIcon": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setContextMenu": True,
        "setIcon": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setToolTip": True,
        "setVisible": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QTabBar": {
        "accessibleTabName": True,
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideEvent": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isTabEnabled": True,
        "isTabVisible": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "minimumTabSizeHint": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveTab": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeTab": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAccessibleTabName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoHide": True,
        "setBackgroundRole": True,
        "setChangeCurrentOnDrag": True,
        "setContextMenuPolicy": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDisabled": True,
        "setDocumentMode": True,
        "setDrawBase": True,
        "setElideMode": True,
        "setEnabled": True,
        "setExpanding": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setMovable": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setSelectionBehaviorOnRemove": True,
        "setShape": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabButton": True,
        "setTabData": True,
        "setTabEnabled": True,
        "setTabIcon": True,
        "setTabText": True,
        "setTabTextColor": True,
        "setTabToolTip": True,
        "setTabVisible": True,
        "setTabWhatsThis": True,
        "setTabletTracking": True,
        "setTabsClosable": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setUsesScrollButtons": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "tabAt": True,
        "tabButton": True,
        "tabData": True,
        "tabIcon": True,
        "tabInserted": True,
        "tabRect": True,
        "tabRemoved": True,
        "tabSizeHint": True,
        "tabText": True,
        "tabTextColor": True,
        "tabToolTip": True,
        "tabWhatsThis": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QTabWidget": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "cornerWidget": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "indexOf": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isTabEnabled": True,
        "isTabVisible": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeTab": True,
        "resizeEvent": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCurrentWidget": True,
        "setCursor": True,
        "setDisabled": True,
        "setDocumentMode": True,
        "setElideMode": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setMovable": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabBar": True,
        "setTabBarAutoHide": True,
        "setTabEnabled": True,
        "setTabIcon": True,
        "setTabPosition": True,
        "setTabShape": True,
        "setTabText": True,
        "setTabToolTip": True,
        "setTabVisible": True,
        "setTabWhatsThis": True,
        "setTabletTracking": True,
        "setTabsClosable": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setUsesScrollButtons": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "tabIcon": True,
        "tabInserted": True,
        "tabRemoved": True,
        "tabText": True,
        "tabToolTip": True,
        "tabWhatsThis": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "widget": True
    },
    "QtWidgets.QTableView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "closePersistentEditor": True,
        "columnAt": True,
        "columnCountChanged": True,
        "columnMoved": True,
        "columnResized": True,
        "columnSpan": True,
        "columnViewportPosition": True,
        "columnWidth": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "editorDestroyed": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideColumn": True,
        "hideRow": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isColumnHidden": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isPersistentEditorOpen": True,
        "isRowHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "openPersistentEditor": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeColumnToContents": True,
        "resizeRowToContents": True,
        "restoreGeometry": True,
        "rowAt": True,
        "rowCountChanged": True,
        "rowHeight": True,
        "rowMoved": True,
        "rowResized": True,
        "rowSpan": True,
        "rowViewportPosition": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "selectColumn": True,
        "selectRow": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setColumnHidden": True,
        "setColumnWidth": True,
        "setContextMenuPolicy": True,
        "setCornerButtonEnabled": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setGridStyle": True,
        "setHidden": True,
        "setHorizontalHeader": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setRootIndex": True,
        "setRowHeight": True,
        "setRowHidden": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setShowGrid": True,
        "setSizeAdjustPolicy": True,
        "setSortingEnabled": True,
        "setSpan": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalHeader": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "setupViewport": True,
        "showColumn": True,
        "showRow": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualRect": True,
        "visualRegionForSelection": True,
        "wheelEvent": True
    },
    "QtWidgets.QTableWidget": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "cellWidget": True,
        "changeEvent": True,
        "closeEditor": True,
        "column": True,
        "columnAt": True,
        "columnCountChanged": True,
        "columnMoved": True,
        "columnResized": True,
        "columnSpan": True,
        "columnViewportPosition": True,
        "columnWidth": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "dropMimeData": True,
        "editItem": True,
        "editorDestroyed": True,
        "findChild": True,
        "findItems": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideColumn": True,
        "hideRow": True,
        "horizontalHeaderItem": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexFromItem": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertColumn": True,
        "insertRow": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isColumnHidden": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isRowHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "item": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "itemFromIndex": True,
        "items": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mimeData": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeCellWidget": True,
        "removeColumn": True,
        "removeEventFilter": True,
        "removeRow": True,
        "resizeColumnToContents": True,
        "resizeRowToContents": True,
        "restoreGeometry": True,
        "row": True,
        "rowAt": True,
        "rowCountChanged": True,
        "rowHeight": True,
        "rowMoved": True,
        "rowResized": True,
        "rowSpan": True,
        "rowViewportPosition": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "scrollToItem": True,
        "selectColumn": True,
        "selectRow": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setCellWidget": True,
        "setColumnCount": True,
        "setColumnHidden": True,
        "setColumnWidth": True,
        "setContextMenuPolicy": True,
        "setCornerButtonEnabled": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setGridStyle": True,
        "setHidden": True,
        "setHorizontalHeader": True,
        "setHorizontalHeaderItem": True,
        "setHorizontalHeaderLabels": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItem": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setItemPrototype": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setRangeSelected": True,
        "setRootIndex": True,
        "setRowCount": True,
        "setRowHeight": True,
        "setRowHidden": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setShowGrid": True,
        "setSizeAdjustPolicy": True,
        "setSortingEnabled": True,
        "setSpan": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVerticalHeader": True,
        "setVerticalHeaderItem": True,
        "setVerticalHeaderLabels": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "setupViewport": True,
        "showColumn": True,
        "showRow": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "sortItems": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "takeHorizontalHeaderItem": True,
        "takeItem": True,
        "takeVerticalHeaderItem": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalHeaderItem": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualColumn": True,
        "visualItemRect": True,
        "visualRect": True,
        "visualRegionForSelection": True,
        "visualRow": True,
        "wheelEvent": True
    },
    "QtWidgets.QTableWidgetItem": {
        "data": True,
        "read": True,
        "setBackground": True,
        "setCheckState": True,
        "setData": True,
        "setFlags": True,
        "setFont": True,
        "setForeground": True,
        "setIcon": True,
        "setSelected": True,
        "setSizeHint": True,
        "setStatusTip": True,
        "setText": True,
        "setTextAlignment": True,
        "setToolTip": True,
        "setWhatsThis": True,
        "write": True
    },
    "QtWidgets.QTapAndHoldGesture": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setGestureCancelPolicy": True,
        "setHotSpot": True,
        "setObjectName": True,
        "setParent": True,
        "setPosition": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QTapGesture": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setGestureCancelPolicy": True,
        "setHotSpot": True,
        "setObjectName": True,
        "setParent": True,
        "setPosition": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QTextBrowser": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "anchorAt": True,
        "append": True,
        "blockSignals": True,
        "canInsertFromMimeData": True,
        "connectNotify": True,
        "create": True,
        "cursorForPosition": True,
        "destroy": True,
        "disconnectNotify": True,
        "doSetSource": True,
        "drawFrame": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "focusOutEvent": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "historyTitle": True,
        "historyUrl": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodEvent": True,
        "insertAction": True,
        "insertActions": True,
        "insertFromMimeData": True,
        "insertHtml": True,
        "insertPlainText": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "loadResource": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "mergeCurrentCharFormat": True,
        "metric": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "print_": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollToAnchor": True,
        "setAcceptDrops": True,
        "setAcceptRichText": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoFormatting": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentCharFormat": True,
        "setCurrentFont": True,
        "setCursor": True,
        "setCursorWidth": True,
        "setDisabled": True,
        "setDocument": True,
        "setDocumentTitle": True,
        "setEnabled": True,
        "setExtraSelections": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setFontFamily": True,
        "setFontItalic": True,
        "setFontPointSize": True,
        "setFontUnderline": True,
        "setFontWeight": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHtml": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLineWrapColumnOrWidth": True,
        "setLineWrapMode": True,
        "setLocale": True,
        "setMarkdown": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOpenExternalLinks": True,
        "setOpenLinks": True,
        "setOverwriteMode": True,
        "setPalette": True,
        "setPlaceholderText": True,
        "setPlainText": True,
        "setProperty": True,
        "setReadOnly": True,
        "setSearchPaths": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabChangesFocus": True,
        "setTabStopDistance": True,
        "setTabStopWidth": True,
        "setTabletTracking": True,
        "setText": True,
        "setTextBackgroundColor": True,
        "setTextColor": True,
        "setTextCursor": True,
        "setTextInteractionFlags": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUndoRedoEnabled": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrapMode": True,
        "setupViewport": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "toMarkdown": True,
        "tr": True,
        "ungrabGesture": True,
        "viewportEvent": True,
        "zoomIn": True,
        "zoomOut": True
    },
    "QtWidgets.QTextEdit": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "anchorAt": True,
        "append": True,
        "blockSignals": True,
        "canInsertFromMimeData": True,
        "connectNotify": True,
        "create": True,
        "cursorForPosition": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "eventFilter": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodEvent": True,
        "insertAction": True,
        "insertActions": True,
        "insertFromMimeData": True,
        "insertHtml": True,
        "insertPlainText": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "loadResource": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "mergeCurrentCharFormat": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "print_": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollToAnchor": True,
        "setAcceptDrops": True,
        "setAcceptRichText": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoFormatting": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentCharFormat": True,
        "setCurrentFont": True,
        "setCursor": True,
        "setCursorWidth": True,
        "setDisabled": True,
        "setDocument": True,
        "setDocumentTitle": True,
        "setEnabled": True,
        "setExtraSelections": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setFontFamily": True,
        "setFontItalic": True,
        "setFontPointSize": True,
        "setFontUnderline": True,
        "setFontWeight": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHtml": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLineWrapColumnOrWidth": True,
        "setLineWrapMode": True,
        "setLocale": True,
        "setMarkdown": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOverwriteMode": True,
        "setPalette": True,
        "setPlaceholderText": True,
        "setPlainText": True,
        "setProperty": True,
        "setReadOnly": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabChangesFocus": True,
        "setTabStopDistance": True,
        "setTabStopWidth": True,
        "setTabletTracking": True,
        "setText": True,
        "setTextBackgroundColor": True,
        "setTextColor": True,
        "setTextCursor": True,
        "setTextInteractionFlags": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUndoRedoEnabled": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrapMode": True,
        "setupViewport": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "toMarkdown": True,
        "tr": True,
        "ungrabGesture": True,
        "viewportEvent": True,
        "zoomIn": True,
        "zoomOut": True
    },
    "QtWidgets.QTimeEdit": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "dateTimeFromText": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "sectionAt": True,
        "sectionText": True,
        "setAccelerated": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlignment": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButtonSymbols": True,
        "setCalendar": True,
        "setCalendarPopup": True,
        "setCalendarWidget": True,
        "setContextMenuPolicy": True,
        "setCorrectionMode": True,
        "setCurrentSection": True,
        "setCurrentSectionIndex": True,
        "setCursor": True,
        "setDate": True,
        "setDateRange": True,
        "setDateTime": True,
        "setDateTimeRange": True,
        "setDisabled": True,
        "setDisplayFormat": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrame": True,
        "setGraphicsEffect": True,
        "setGroupSeparatorShown": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setKeyboardTracking": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineEdit": True,
        "setLocale": True,
        "setMaximumDate": True,
        "setMaximumDateTime": True,
        "setMaximumHeight": True,
        "setMaximumTime": True,
        "setMaximumWidth": True,
        "setMinimumDate": True,
        "setMinimumDateTime": True,
        "setMinimumHeight": True,
        "setMinimumTime": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setReadOnly": True,
        "setSelectedSection": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSpecialValueText": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setTime": True,
        "setTimeRange": True,
        "setTimeSpec": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWrapping": True,
        "stackUnder": True,
        "startTimer": True,
        "stepBy": True,
        "testAttribute": True,
        "textFromDateTime": True,
        "tr": True,
        "ungrabGesture": True,
        "validate": True
    },
    "QtWidgets.QToolBar": {
        "actionGeometry": True,
        "addActions": True,
        "addWidget": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertSeparator": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isAreaAllowed": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAllowedAreas": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFloatable": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setMovable": True,
        "setObjectName": True,
        "setOrientation": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolButtonStyle": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "widgetForAction": True
    },
    "QtWidgets.QToolBox": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "indexOf": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isItemEnabled": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemIcon": True,
        "itemInserted": True,
        "itemRemoved": True,
        "itemText": True,
        "itemToolTip": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeItem": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCurrentIndex": True,
        "setCurrentWidget": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setItemEnabled": True,
        "setItemIcon": True,
        "setItemText": True,
        "setItemToolTip": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "widget": True
    },
    "QtWidgets.QToolButton": {
        "actionEvent": True,
        "addAction": True,
        "addActions": True,
        "animateClick": True,
        "blockSignals": True,
        "changeEvent": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "enterEvent": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hitButton": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "leaveEvent": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "paintEvent": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setArrowType": True,
        "setAttribute": True,
        "setAutoExclusive": True,
        "setAutoFillBackground": True,
        "setAutoRaise": True,
        "setAutoRepeat": True,
        "setAutoRepeatDelay": True,
        "setAutoRepeatInterval": True,
        "setBackgroundRole": True,
        "setCheckable": True,
        "setChecked": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDefaultAction": True,
        "setDisabled": True,
        "setDown": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setIcon": True,
        "setIconSize": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMenu": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPopupMode": True,
        "setProperty": True,
        "setShortcut": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setText": True,
        "setToolButtonStyle": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "timerEvent": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QTreeView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "closePersistentEditor": True,
        "collapse": True,
        "columnAt": True,
        "columnCountChanged": True,
        "columnResized": True,
        "columnViewportPosition": True,
        "columnWidth": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawBranches": True,
        "drawFrame": True,
        "drawRow": True,
        "drawTree": True,
        "editorDestroyed": True,
        "expand": True,
        "expandRecursively": True,
        "expandToDepth": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideColumn": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAbove": True,
        "indexAt": True,
        "indexBelow": True,
        "indexRowSizeHint": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isColumnHidden": True,
        "isEnabledTo": True,
        "isExpanded": True,
        "isFirstColumnSpanned": True,
        "isIndexHidden": True,
        "isPersistentEditorOpen": True,
        "isRowHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "openPersistentEditor": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "resizeColumnToContents": True,
        "restoreGeometry": True,
        "rowHeight": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "rowsRemoved": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAllColumnsShowFocus": True,
        "setAlternatingRowColors": True,
        "setAnimated": True,
        "setAttribute": True,
        "setAutoExpandDelay": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setColumnHidden": True,
        "setColumnWidth": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setExpanded": True,
        "setExpandsOnDoubleClick": True,
        "setFirstColumnSpanned": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHeader": True,
        "setHeaderHidden": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndentation": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setItemsExpandable": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setRootIndex": True,
        "setRootIsDecorated": True,
        "setRowHidden": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setSortingEnabled": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTreePosition": True,
        "setUniformRowHeights": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "setupViewport": True,
        "showColumn": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualRect": True,
        "visualRegionForSelection": True,
        "wheelEvent": True
    },
    "QtWidgets.QTreeWidget": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "addTopLevelItem": True,
        "addTopLevelItems": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "collapse": True,
        "collapseItem": True,
        "columnAt": True,
        "columnCountChanged": True,
        "columnResized": True,
        "columnViewportPosition": True,
        "columnWidth": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawBranches": True,
        "drawFrame": True,
        "drawRow": True,
        "drawTree": True,
        "dropMimeData": True,
        "editItem": True,
        "editorDestroyed": True,
        "expand": True,
        "expandItem": True,
        "expandRecursively": True,
        "expandToDepth": True,
        "findChild": True,
        "findItems": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "hideColumn": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAbove": True,
        "indexAt": True,
        "indexBelow": True,
        "indexFromItem": True,
        "indexOfTopLevelItem": True,
        "indexRowSizeHint": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "insertTopLevelItem": True,
        "insertTopLevelItems": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isColumnHidden": True,
        "isEnabledTo": True,
        "isExpanded": True,
        "isFirstColumnSpanned": True,
        "isFirstItemColumnSpanned": True,
        "isIndexHidden": True,
        "isRowHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemAbove": True,
        "itemBelow": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "itemFromIndex": True,
        "itemWidget": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "mimeData": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removeItemWidget": True,
        "resizeColumnToContents": True,
        "restoreGeometry": True,
        "rowHeight": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "rowsRemoved": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "scrollToItem": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAllColumnsShowFocus": True,
        "setAlternatingRowColors": True,
        "setAnimated": True,
        "setAttribute": True,
        "setAutoExpandDelay": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setColumnCount": True,
        "setColumnHidden": True,
        "setColumnWidth": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEnabled": True,
        "setExpanded": True,
        "setExpandsOnDoubleClick": True,
        "setFirstColumnSpanned": True,
        "setFirstItemColumnSpanned": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setHeader": True,
        "setHeaderHidden": True,
        "setHeaderItem": True,
        "setHeaderLabel": True,
        "setHeaderLabels": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndentation": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setItemWidget": True,
        "setItemsExpandable": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setRootIndex": True,
        "setRootIsDecorated": True,
        "setRowHidden": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setSortingEnabled": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setTreePosition": True,
        "setUniformRowHeights": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "setupViewport": True,
        "showColumn": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "sortItems": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "takeTopLevelItem": True,
        "testAttribute": True,
        "topLevelItem": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualItemRect": True,
        "visualRect": True,
        "visualRegionForSelection": True,
        "wheelEvent": True
    },
    "QtWidgets.QTreeWidgetItem": {
        "addChild": True,
        "addChildren": True,
        "background": True,
        "checkState": True,
        "child": True,
        "data": True,
        "font": True,
        "foreground": True,
        "icon": True,
        "indexOfChild": True,
        "insertChild": True,
        "insertChildren": True,
        "read": True,
        "removeChild": True,
        "setBackground": True,
        "setCheckState": True,
        "setChildIndicatorPolicy": True,
        "setData": True,
        "setDisabled": True,
        "setExpanded": True,
        "setFirstColumnSpanned": True,
        "setFlags": True,
        "setFont": True,
        "setForeground": True,
        "setHidden": True,
        "setIcon": True,
        "setSelected": True,
        "setSizeHint": True,
        "setStatusTip": True,
        "setText": True,
        "setTextAlignment": True,
        "setToolTip": True,
        "setWhatsThis": True,
        "sizeHint": True,
        "sortChildren": True,
        "statusTip": True,
        "takeChild": True,
        "text": True,
        "textAlignment": True,
        "toolTip": True,
        "whatsThis": True,
        "write": True
    },
    "QtWidgets.QUndoCommand": {
        "child": True,
        "mergeWith": True,
        "setObsolete": True,
        "setText": True
    },
    "QtWidgets.QUndoGroup": {
        "addStack": True,
        "blockSignals": True,
        "connectNotify": True,
        "createRedoAction": True,
        "createUndoAction": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeStack": True,
        "setActiveStack": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QUndoStack": {
        "beginMacro": True,
        "blockSignals": True,
        "command": True,
        "connectNotify": True,
        "createRedoAction": True,
        "createUndoAction": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "push": True,
        "receivers": True,
        "removeEventFilter": True,
        "setActive": True,
        "setIndex": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setUndoLimit": True,
        "startTimer": True,
        "text": True,
        "tr": True
    },
    "QtWidgets.QUndoView": {
        "addAction": True,
        "addActions": True,
        "addScrollBarWidget": True,
        "blockSignals": True,
        "changeEvent": True,
        "closeEditor": True,
        "closePersistentEditor": True,
        "commitData": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "currentChanged": True,
        "dataChanged": True,
        "destroy": True,
        "disconnectNotify": True,
        "drawFrame": True,
        "editorDestroyed": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "horizontalScrollbarAction": True,
        "horizontalScrollbarValueChanged": True,
        "indexAt": True,
        "indexWidget": True,
        "inherits": True,
        "initPainter": True,
        "initStyleOption": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isIndexHidden": True,
        "isPersistentEditorOpen": True,
        "isRowHidden": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "itemDelegateForColumn": True,
        "itemDelegateForRow": True,
        "keyboardSearch": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveCursor": True,
        "moveToThread": True,
        "nativeEvent": True,
        "openPersistentEditor": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "rectForIndex": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "rowsAboutToBeRemoved": True,
        "rowsInserted": True,
        "scrollBarWidgets": True,
        "scrollContentsBy": True,
        "scrollDirtyRegion": True,
        "scrollTo": True,
        "selectionChanged": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAlternatingRowColors": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setAutoScroll": True,
        "setAutoScrollMargin": True,
        "setBackgroundRole": True,
        "setBatchSize": True,
        "setCleanIcon": True,
        "setContextMenuPolicy": True,
        "setCornerWidget": True,
        "setCurrentIndex": True,
        "setCursor": True,
        "setDefaultDropAction": True,
        "setDirtyRegion": True,
        "setDisabled": True,
        "setDragDropMode": True,
        "setDragDropOverwriteMode": True,
        "setDragEnabled": True,
        "setDropIndicatorShown": True,
        "setEditTriggers": True,
        "setEmptyLabel": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFlow": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setFrameRect": True,
        "setFrameShadow": True,
        "setFrameShape": True,
        "setFrameStyle": True,
        "setGraphicsEffect": True,
        "setGridSize": True,
        "setGroup": True,
        "setHidden": True,
        "setHorizontalScrollBar": True,
        "setHorizontalScrollBarPolicy": True,
        "setHorizontalScrollMode": True,
        "setIconSize": True,
        "setIndexWidget": True,
        "setInputMethodHints": True,
        "setItemAlignment": True,
        "setItemDelegate": True,
        "setItemDelegateForColumn": True,
        "setItemDelegateForRow": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLayoutMode": True,
        "setLineWidth": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMidLineWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModel": True,
        "setModelColumn": True,
        "setMouseTracking": True,
        "setMovement": True,
        "setObjectName": True,
        "setPalette": True,
        "setPositionForIndex": True,
        "setProperty": True,
        "setResizeMode": True,
        "setRootIndex": True,
        "setRowHidden": True,
        "setSelection": True,
        "setSelectionBehavior": True,
        "setSelectionMode": True,
        "setSelectionModel": True,
        "setSelectionRectVisible": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSizeAdjustPolicy": True,
        "setSpacing": True,
        "setStack": True,
        "setState": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabKeyNavigation": True,
        "setTabletTracking": True,
        "setTextElideMode": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUniformItemSizes": True,
        "setUpdatesEnabled": True,
        "setVerticalScrollBar": True,
        "setVerticalScrollBarPolicy": True,
        "setVerticalScrollMode": True,
        "setViewMode": True,
        "setViewport": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWordWrap": True,
        "setWrapping": True,
        "setupViewport": True,
        "sizeHintForColumn": True,
        "sizeHintForIndex": True,
        "sizeHintForRow": True,
        "stackUnder": True,
        "startDrag": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True,
        "verticalScrollbarAction": True,
        "verticalScrollbarValueChanged": True,
        "visualRect": True,
        "visualRegionForSelection": True
    },
    "QtWidgets.QVBoxLayout": {
        "addChildLayout": True,
        "addChildWidget": True,
        "addItem": True,
        "addLayout": True,
        "addSpacerItem": True,
        "addSpacing": True,
        "addStretch": True,
        "addStrut": True,
        "alignmentRect": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "heightForWidth": True,
        "inherits": True,
        "insertItem": True,
        "insertLayout": True,
        "insertSpacerItem": True,
        "insertSpacing": True,
        "insertStretch": True,
        "insertWidget": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemAt": True,
        "killTimer": True,
        "minimumHeightForWidth": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeItem": True,
        "removeWidget": True,
        "replaceWidget": True,
        "setDirection": True,
        "setEnabled": True,
        "setGeometry": True,
        "setMenuBar": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSizeConstraint": True,
        "setSpacing": True,
        "setStretch": True,
        "startTimer": True,
        "stretch": True,
        "takeAt": True,
        "totalHeightForWidth": True,
        "tr": True,
        "widgetEvent": True
    },
    "QtWidgets.QWidget": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setTabletTracking": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QWidgetAction": {
        "blockSignals": True,
        "connectNotify": True,
        "createWidget": True,
        "deleteWidget": True,
        "disconnectNotify": True,
        "event": True,
        "eventFilter": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "releaseWidget": True,
        "removeEventFilter": True,
        "requestWidget": True,
        "setActionGroup": True,
        "setAutoRepeat": True,
        "setCheckable": True,
        "setChecked": True,
        "setData": True,
        "setDefaultWidget": True,
        "setDisabled": True,
        "setEnabled": True,
        "setFont": True,
        "setIcon": True,
        "setIconText": True,
        "setIconVisibleInMenu": True,
        "setMenu": True,
        "setMenuRole": True,
        "setObjectName": True,
        "setParent": True,
        "setPriority": True,
        "setProperty": True,
        "setSeparator": True,
        "setShortcut": True,
        "setShortcutContext": True,
        "setShortcutVisibleInContextMenu": True,
        "setStatusTip": True,
        "setText": True,
        "setToolTip": True,
        "setVisible": True,
        "setWhatsThis": True,
        "showStatusText": True,
        "startTimer": True,
        "tr": True
    },
    "QtWidgets.QWidgetItem": {
        "heightForWidth": True,
        "minimumHeightForWidth": True,
        "setAlignment": True,
        "setGeometry": True
    },
    "QtWidgets.QWizard": {
        "addAction": True,
        "addActions": True,
        "addPage": True,
        "blockSignals": True,
        "button": True,
        "buttonText": True,
        "cleanupPage": True,
        "closeEvent": True,
        "connectNotify": True,
        "contextMenuEvent": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "done": True,
        "eventFilter": True,
        "field": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "hasVisitedPage": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "initializePage": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "keyPressEvent": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "page": True,
        "pixmap": True,
        "property": True,
        "receivers": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "removePage": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButton": True,
        "setButtonLayout": True,
        "setButtonText": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDefaultProperty": True,
        "setDisabled": True,
        "setEnabled": True,
        "setField": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setModal": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setOption": True,
        "setOptions": True,
        "setPage": True,
        "setPalette": True,
        "setPixmap": True,
        "setProperty": True,
        "setResult": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setSideWidget": True,
        "setSizeGripEnabled": True,
        "setStartId": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setSubTitleFormat": True,
        "setTabletTracking": True,
        "setTitleFormat": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "setWizardStyle": True,
        "showEvent": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "testOption": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtWidgets.QWizardPage": {
        "addAction": True,
        "addActions": True,
        "blockSignals": True,
        "buttonText": True,
        "connectNotify": True,
        "create": True,
        "destroy": True,
        "disconnectNotify": True,
        "field": True,
        "findChild": True,
        "focusNextPrevChild": True,
        "grab": True,
        "grabGesture": True,
        "grabShortcut": True,
        "heightForWidth": True,
        "inherits": True,
        "initPainter": True,
        "inputMethodQuery": True,
        "insertAction": True,
        "insertActions": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isEnabledTo": True,
        "isSignalConnected": True,
        "isVisibleTo": True,
        "killTimer": True,
        "mapFrom": True,
        "mapFromGlobal": True,
        "mapFromParent": True,
        "mapTo": True,
        "mapToGlobal": True,
        "mapToParent": True,
        "metric": True,
        "moveToThread": True,
        "nativeEvent": True,
        "overrideWindowFlags": True,
        "overrideWindowState": True,
        "pixmap": True,
        "property": True,
        "receivers": True,
        "registerField": True,
        "releaseShortcut": True,
        "removeAction": True,
        "removeEventFilter": True,
        "restoreGeometry": True,
        "setAcceptDrops": True,
        "setAccessibleDescription": True,
        "setAccessibleName": True,
        "setAttribute": True,
        "setAutoFillBackground": True,
        "setBackgroundRole": True,
        "setButtonText": True,
        "setCommitPage": True,
        "setContextMenuPolicy": True,
        "setCursor": True,
        "setDisabled": True,
        "setEnabled": True,
        "setField": True,
        "setFinalPage": True,
        "setFixedHeight": True,
        "setFixedWidth": True,
        "setFocusPolicy": True,
        "setFocusProxy": True,
        "setFont": True,
        "setForegroundRole": True,
        "setGraphicsEffect": True,
        "setHidden": True,
        "setInputMethodHints": True,
        "setLayout": True,
        "setLayoutDirection": True,
        "setLocale": True,
        "setMaximumHeight": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumWidth": True,
        "setMouseTracking": True,
        "setObjectName": True,
        "setPalette": True,
        "setPixmap": True,
        "setProperty": True,
        "setShortcutAutoRepeat": True,
        "setShortcutEnabled": True,
        "setStatusTip": True,
        "setStyle": True,
        "setStyleSheet": True,
        "setSubTitle": True,
        "setTabletTracking": True,
        "setTitle": True,
        "setToolTip": True,
        "setToolTipDuration": True,
        "setUpdatesEnabled": True,
        "setVisible": True,
        "setWhatsThis": True,
        "setWindowFilePath": True,
        "setWindowFlag": True,
        "setWindowFlags": True,
        "setWindowIcon": True,
        "setWindowIconText": True,
        "setWindowModality": True,
        "setWindowModified": True,
        "setWindowOpacity": True,
        "setWindowRole": True,
        "setWindowState": True,
        "setWindowTitle": True,
        "stackUnder": True,
        "startTimer": True,
        "testAttribute": True,
        "tr": True,
        "ungrabGesture": True
    },
    "QtCore.QAbstractAnimation": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setDirection": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "start": True,
        "startTimer": True,
        "tr": True,
        "updateCurrentTime": True,
        "updateDirection": True,
        "updateState": True
    },
    "QtCore.QAbstractEventDispatcher": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "filterNativeEvent": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "installNativeEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "processEvents": True,
        "property": True,
        "receivers": True,
        "registerEventNotifier": True,
        "registerSocketNotifier": True,
        "registeredTimers": True,
        "remainingTime": True,
        "removeEventFilter": True,
        "removeNativeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True,
        "unregisterEventNotifier": True,
        "unregisterSocketNotifier": True,
        "unregisterTimer": True,
        "unregisterTimers": True
    },
    "QtCore.QAbstractItemModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QAbstractListModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QAbstractNativeEventFilter": {
        "nativeEventFilter": True
    },
    "QtCore.QAbstractProxyModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "mapFromSource": True,
        "mapSelectionFromSource": True,
        "mapSelectionToSource": True,
        "mapToSource": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSourceModel": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QAbstractState": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QAbstractTableModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QAbstractTransition": {
        "addAnimation": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setTargetState": True,
        "setTargetStates": True,
        "setTransitionType": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QAnimationGroup": {
        "addAnimation": True,
        "animationAt": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "indexOfAnimation": True,
        "inherits": True,
        "insertAnimation": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setDirection": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "start": True,
        "startTimer": True,
        "takeAnimation": True,
        "tr": True,
        "updateCurrentTime": True,
        "updateDirection": True,
        "updateState": True
    },
    "QtCore.QBasicTimer": {
        "swap": True
    },
    "QtCore.QBitArray": {
        "at": True,
        "clearBit": True,
        "resize": True,
        "swap": True,
        "testBit": True,
        "toggleBit": True,
        "truncate": True
    },
    "QtCore.QBuffer": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "getChar": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "open": True,
        "peek": True,
        "property": True,
        "putChar": True,
        "read": True,
        "readData": True,
        "readLine": True,
        "readLineData": True,
        "receivers": True,
        "removeEventFilter": True,
        "seek": True,
        "setBuffer": True,
        "setCurrentReadChannel": True,
        "setCurrentWriteChannel": True,
        "setData": True,
        "setErrorString": True,
        "setObjectName": True,
        "setOpenMode": True,
        "setParent": True,
        "setProperty": True,
        "setTextModeEnabled": True,
        "skip": True,
        "startTimer": True,
        "tr": True,
        "ungetChar": True,
        "waitForBytesWritten": True,
        "waitForReadyRead": True,
        "write": True,
        "writeData": True
    },
    "QtCore.QByteArray": {
        "at": True,
        "chop": True,
        "chopped": True,
        "fill": True,
        "indexOf": True,
        "lastIndexOf": True,
        "left": True,
        "leftJustified": True,
        "mid": True,
        "remove": True,
        "repeated": True,
        "reserve": True,
        "resize": True,
        "right": True,
        "rightJustified": True,
        "split": True,
        "swap": True,
        "toInt": True,
        "toLong": True,
        "toLongLong": True,
        "toPercentEncoding": True,
        "toShort": True,
        "toUInt": True,
        "toULong": True,
        "toULongLong": True,
        "toUShort": True,
        "truncate": True
    },
    "QtCore.QByteArrayMatcher": {
        "setPattern": True
    },
    "QtCore.QCalendar": {
        "dayOfWeek": True,
        "daysInMonth": True,
        "daysInYear": True,
        "isDateValid": True,
        "isLeapYear": True,
        "monthName": True,
        "monthsInYear": True,
        "partsFromDate": True,
        "standaloneMonthName": True,
        "standaloneWeekDayName": True,
        "weekDayName": True
    },
    "QtCore.QCborStreamReader": {
        "next": True,
        "setDevice": True
    },
    "QtCore.QCborStreamWriter": {
        "setDevice": True
    },
    "QtCore.QChildEvent": {
        "setAccepted": True
    },
    "QtCore.QCollator": {
        "setCaseSensitivity": True,
        "setIgnorePunctuation": True,
        "setLocale": True,
        "setNumericMode": True,
        "sortKey": True,
        "swap": True
    },
    "QtCore.QCollatorSortKey": {
        "compare": True,
        "swap": True
    },
    "QtCore.QCommandLineOption": {
        "setDefaultValue": True,
        "setDefaultValues": True,
        "setDescription": True,
        "setFlags": True,
        "setHidden": True,
        "setValueName": True,
        "swap": True
    },
    "QtCore.QCommandLineParser": {
        "addOption": True,
        "addOptions": True,
        "addPositionalArgument": True,
        "parse": True,
        "setApplicationDescription": True,
        "setOptionsAfterPositionalArgumentsMode": True,
        "setSingleDashWordOptionMode": True,
        "showHelp": True
    },
    "QtCore.QConcatenateTablesProxyModel": {
        "addSourceModel": True,
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "mapFromSource": True,
        "mapToSource": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "removeSourceModel": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QCoreApplication": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "installNativeEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "notify": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeNativeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QDataStream": {
        "readRawData": True,
        "setByteOrder": True,
        "setDevice": True,
        "setFloatingPointPrecision": True,
        "setStatus": True,
        "setVersion": True,
        "skipRawData": True,
        "writeBool": True,
        "writeDouble": True,
        "writeFloat": True,
        "writeInt16": True,
        "writeInt32": True,
        "writeInt64": True,
        "writeInt8": True,
        "writeQString": True,
        "writeQStringList": True,
        "writeQVariant": True,
        "writeRawData": True,
        "writeString": True,
        "writeUInt16": True,
        "writeUInt32": True,
        "writeUInt64": True,
        "writeUInt8": True
    },
    "QtCore.QDate": {
        "addDays": True,
        "daysTo": True
    },
    "QtCore.QDateTime": {
        "addDays": True,
        "addMSecs": True,
        "addMonths": True,
        "addSecs": True,
        "addYears": True,
        "daysTo": True,
        "msecsTo": True,
        "secsTo": True,
        "setDate": True,
        "setMSecsSinceEpoch": True,
        "setOffsetFromUtc": True,
        "setSecsSinceEpoch": True,
        "setTime": True,
        "setTimeSpec": True,
        "setTimeZone": True,
        "setTime_t": True,
        "swap": True,
        "toOffsetFromUtc": True,
        "toTimeSpec": True,
        "toTimeZone": True
    },
    "QtCore.QDeadlineTimer": {
        "setDeadline": True,
        "setPreciseDeadline": True,
        "setPreciseRemainingTime": True,
        "setRemainingTime": True,
        "setTimerType": True,
        "swap": True
    },
    "QtCore.QDir": {
        "absoluteFilePath": True,
        "cd": True,
        "filePath": True,
        "isEmpty": True,
        "mkdir": True,
        "mkpath": True,
        "relativeFilePath": True,
        "remove": True,
        "rename": True,
        "rmdir": True,
        "rmpath": True,
        "setFilter": True,
        "setNameFilters": True,
        "setPath": True,
        "setSorting": True,
        "swap": True
    },
    "QtCore.QDynamicPropertyChangeEvent": {
        "setAccepted": True
    },
    "QtCore.QEasingCurve": {
        "addCubicBezierSegment": True,
        "addTCBSegment": True,
        "setAmplitude": True,
        "setCustomType": True,
        "setOvershoot": True,
        "setPeriod": True,
        "setType": True,
        "swap": True,
        "valueForProgress": True
    },
    "QtCore.QElapsedTimer": {
        "hasExpired": True,
        "msecsTo": True,
        "secsTo": True
    },
    "QtCore.QEvent": {
        "setAccepted": True
    },
    "QtCore.QEventLoop": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "exec_": True,
        "exit": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QEventTransition": {
        "addAnimation": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setEventSource": True,
        "setEventType": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setTargetState": True,
        "setTargetStates": True,
        "setTransitionType": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QFile": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "fileTime": True,
        "findChild": True,
        "getChar": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "map": True,
        "moveToThread": True,
        "peek": True,
        "property": True,
        "putChar": True,
        "read": True,
        "readData": True,
        "readLine": True,
        "readLineData": True,
        "receivers": True,
        "removeEventFilter": True,
        "seek": True,
        "setCurrentReadChannel": True,
        "setCurrentWriteChannel": True,
        "setErrorString": True,
        "setFileName": True,
        "setFileTime": True,
        "setObjectName": True,
        "setOpenMode": True,
        "setParent": True,
        "setProperty": True,
        "setTextModeEnabled": True,
        "skip": True,
        "startTimer": True,
        "tr": True,
        "ungetChar": True,
        "unmap": True,
        "waitForBytesWritten": True,
        "waitForReadyRead": True,
        "write": True,
        "writeData": True
    },
    "QtCore.QFileDevice": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "fileTime": True,
        "findChild": True,
        "getChar": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "map": True,
        "moveToThread": True,
        "open": True,
        "peek": True,
        "property": True,
        "putChar": True,
        "read": True,
        "readData": True,
        "readLine": True,
        "readLineData": True,
        "receivers": True,
        "removeEventFilter": True,
        "resize": True,
        "seek": True,
        "setCurrentReadChannel": True,
        "setCurrentWriteChannel": True,
        "setErrorString": True,
        "setFileTime": True,
        "setObjectName": True,
        "setOpenMode": True,
        "setParent": True,
        "setPermissions": True,
        "setProperty": True,
        "setTextModeEnabled": True,
        "skip": True,
        "startTimer": True,
        "tr": True,
        "ungetChar": True,
        "unmap": True,
        "waitForBytesWritten": True,
        "waitForReadyRead": True,
        "write": True,
        "writeData": True
    },
    "QtCore.QFileInfo": {
        "setCaching": True,
        "swap": True
    },
    "QtCore.QFileSelector": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setExtraSelectors": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QFileSystemWatcher": {
        "addPath": True,
        "addPaths": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removePath": True,
        "removePaths": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QFinalState": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QHistoryState": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setDefaultState": True,
        "setDefaultTransition": True,
        "setHistoryType": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QIODevice": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "getChar": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "open": True,
        "peek": True,
        "property": True,
        "putChar": True,
        "read": True,
        "readData": True,
        "readLine": True,
        "readLineData": True,
        "receivers": True,
        "removeEventFilter": True,
        "seek": True,
        "setCurrentReadChannel": True,
        "setCurrentWriteChannel": True,
        "setErrorString": True,
        "setObjectName": True,
        "setOpenMode": True,
        "setParent": True,
        "setProperty": True,
        "setTextModeEnabled": True,
        "skip": True,
        "startTimer": True,
        "tr": True,
        "ungetChar": True,
        "waitForBytesWritten": True,
        "waitForReadyRead": True,
        "write": True,
        "writeData": True
    },
    "QtCore.QIdentityProxyModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "mapFromSource": True,
        "mapSelectionFromSource": True,
        "mapSelectionToSource": True,
        "mapToSource": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSourceModel": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QItemSelection": {
        "contains": True,
        "indexOf": True,
        "insert": True,
        "lastIndexOf": True,
        "merge": True,
        "move": True,
        "prepend": True,
        "removeAll": True,
        "removeAt": True,
        "replace": True,
        "select": True,
        "takeAt": True
    },
    "QtCore.QItemSelectionModel": {
        "blockSignals": True,
        "columnIntersectsSelection": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "emitSelectionChanged": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isColumnSelected": True,
        "isRowSelected": True,
        "isSelected": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "rowIntersectsSelection": True,
        "selectedColumns": True,
        "selectedRows": True,
        "setCurrentIndex": True,
        "setModel": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QItemSelectionRange": {
        "intersected": True,
        "intersects": True,
        "swap": True
    },
    "QtCore.QJsonDocument": {
        "setArray": True,
        "setObject": True,
        "swap": True
    },
    "QtCore.QJsonValue": {
        "swap": True,
        "toBool": True,
        "toDouble": True,
        "toInt": True
    },
    "QtCore.QLine": {
        "setLine": True,
        "setP1": True,
        "setP2": True,
        "setPoints": True
    },
    "QtCore.QLineF": {
        "angleTo": True,
        "intersect": True,
        "intersects": True,
        "pointAt": True,
        "setAngle": True,
        "setLength": True,
        "setLine": True,
        "setP1": True,
        "setP2": True,
        "setPoints": True
    },
    "QtCore.QLocale": {
        "createSeparatedList": True,
        "currencySymbol": True,
        "dateFormat": True,
        "dateTimeFormat": True,
        "dayName": True,
        "formattedDataSize": True,
        "monthName": True,
        "setNumberOptions": True,
        "standaloneDayName": True,
        "standaloneMonthName": True,
        "swap": True,
        "timeFormat": True,
        "toDouble": True,
        "toFloat": True,
        "toInt": True,
        "toLongLong": True,
        "toLower": True,
        "toShort": True,
        "toUInt": True,
        "toULongLong": True,
        "toUShort": True,
        "toUpper": True
    },
    "QtCore.QLockFile": {
        "setStaleLockTime": True,
        "tryLock": True
    },
    "QtCore.QMargins": {
        "setBottom": True,
        "setLeft": True,
        "setRight": True,
        "setTop": True
    },
    "QtCore.QMarginsF": {
        "setBottom": True,
        "setLeft": True,
        "setRight": True,
        "setTop": True
    },
    "QtCore.QMessageAuthenticationCode": {
        "setKey": True
    },
    "QtCore.QMetaEnum": {
        "key": True,
        "keyToValue": True,
        "keysToValue": True,
        "value": True,
        "valueToKey": True,
        "valueToKeys": True
    },
    "QtCore.QMetaMethod": {
        "parameterType": True
    },
    "QtCore.QMetaObject": {
        "classInfo": True,
        "constructor": True,
        "enumerator": True,
        "indexOfClassInfo": True,
        "indexOfConstructor": True,
        "indexOfEnumerator": True,
        "indexOfMethod": True,
        "indexOfProperty": True,
        "indexOfSignal": True,
        "indexOfSlot": True,
        "inherits": True,
        "method": True,
        "property": True
    },
    "QtCore.QMetaProperty": {
        "isDesignable": True,
        "isScriptable": True,
        "isStored": True,
        "isUser": True,
        "read": True,
        "reset": True,
        "write": True
    },
    "QtCore.QMimeData": {
        "blockSignals": True,
        "connectNotify": True,
        "data": True,
        "disconnectNotify": True,
        "findChild": True,
        "hasFormat": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeFormat": True,
        "retrieveData": True,
        "setColorData": True,
        "setData": True,
        "setHtml": True,
        "setImageData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setText": True,
        "setUrls": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QMimeDatabase": {
        "mimeTypeForName": True,
        "mimeTypeForUrl": True,
        "mimeTypesForFileName": True,
        "suffixForFileName": True
    },
    "QtCore.QMimeType": {
        "inherits": True,
        "swap": True
    },
    "QtCore.QModelIndex": {
        "child": True,
        "data": True,
        "sibling": True,
        "siblingAtColumn": True,
        "siblingAtRow": True
    },
    "QtCore.QObject": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QParallelAnimationGroup": {
        "addAnimation": True,
        "animationAt": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "indexOfAnimation": True,
        "inherits": True,
        "insertAnimation": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setDirection": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "start": True,
        "startTimer": True,
        "takeAnimation": True,
        "tr": True,
        "updateCurrentTime": True,
        "updateDirection": True,
        "updateState": True
    },
    "QtCore.QPauseAnimation": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setDirection": True,
        "setDuration": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "start": True,
        "startTimer": True,
        "tr": True,
        "updateCurrentTime": True,
        "updateDirection": True,
        "updateState": True
    },
    "QtCore.QPersistentModelIndex": {
        "child": True,
        "data": True,
        "sibling": True,
        "swap": True
    },
    "QtCore.QPluginLoader": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setFileName": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QPoint": {
        "setX": True,
        "setY": True
    },
    "QtCore.QPointF": {
        "setX": True,
        "setY": True
    },
    "QtCore.QProcess": {
        "blockSignals": True,
        "closeReadChannel": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "getChar": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "open": True,
        "peek": True,
        "property": True,
        "putChar": True,
        "read": True,
        "readData": True,
        "readLine": True,
        "readLineData": True,
        "receivers": True,
        "removeEventFilter": True,
        "seek": True,
        "setArguments": True,
        "setCurrentReadChannel": True,
        "setCurrentWriteChannel": True,
        "setErrorString": True,
        "setInputChannelMode": True,
        "setObjectName": True,
        "setOpenMode": True,
        "setParent": True,
        "setProcessChannelMode": True,
        "setProcessEnvironment": True,
        "setProcessState": True,
        "setProgram": True,
        "setProperty": True,
        "setReadChannel": True,
        "setStandardErrorFile": True,
        "setStandardInputFile": True,
        "setStandardOutputFile": True,
        "setStandardOutputProcess": True,
        "setTextModeEnabled": True,
        "setWorkingDirectory": True,
        "skip": True,
        "startTimer": True,
        "tr": True,
        "ungetChar": True,
        "waitForBytesWritten": True,
        "waitForFinished": True,
        "waitForReadyRead": True,
        "waitForStarted": True,
        "write": True,
        "writeData": True
    },
    "QtCore.QProcessEnvironment": {
        "contains": True,
        "remove": True,
        "swap": True,
        "value": True
    },
    "QtCore.QPropertyAnimation": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "interpolated": True,
        "isSignalConnected": True,
        "keyValueAt": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setDirection": True,
        "setDuration": True,
        "setEasingCurve": True,
        "setEndValue": True,
        "setKeyValueAt": True,
        "setKeyValues": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "setPropertyName": True,
        "setStartValue": True,
        "setTargetObject": True,
        "start": True,
        "startTimer": True,
        "tr": True,
        "updateCurrentTime": True,
        "updateCurrentValue": True,
        "updateDirection": True,
        "updateState": True
    },
    "QtCore.QRandomGenerator": {
        "discard": True,
        "seed": True
    },
    "QtCore.QRect": {
        "adjust": True,
        "adjusted": True,
        "intersected": True,
        "intersects": True,
        "marginsAdded": True,
        "marginsRemoved": True,
        "moveBottom": True,
        "moveBottomLeft": True,
        "moveBottomRight": True,
        "moveCenter": True,
        "moveLeft": True,
        "moveRight": True,
        "moveTop": True,
        "moveTopLeft": True,
        "moveTopRight": True,
        "setBottom": True,
        "setBottomLeft": True,
        "setBottomRight": True,
        "setCoords": True,
        "setHeight": True,
        "setLeft": True,
        "setRect": True,
        "setRight": True,
        "setSize": True,
        "setTop": True,
        "setTopLeft": True,
        "setTopRight": True,
        "setWidth": True,
        "setX": True,
        "setY": True,
        "united": True
    },
    "QtCore.QRectF": {
        "adjust": True,
        "adjusted": True,
        "intersected": True,
        "intersects": True,
        "marginsAdded": True,
        "marginsRemoved": True,
        "moveBottom": True,
        "moveBottomLeft": True,
        "moveBottomRight": True,
        "moveCenter": True,
        "moveLeft": True,
        "moveRight": True,
        "moveTop": True,
        "moveTopLeft": True,
        "moveTopRight": True,
        "setBottom": True,
        "setBottomLeft": True,
        "setBottomRight": True,
        "setCoords": True,
        "setHeight": True,
        "setLeft": True,
        "setRect": True,
        "setRight": True,
        "setSize": True,
        "setTop": True,
        "setTopLeft": True,
        "setTopRight": True,
        "setWidth": True,
        "setX": True,
        "setY": True,
        "united": True
    },
    "QtCore.QRegExp": {
        "cap": True,
        "exactMatch": True,
        "indexIn": True,
        "lastIndexIn": True,
        "pos": True,
        "setCaseSensitivity": True,
        "setMinimal": True,
        "setPattern": True,
        "setPatternSyntax": True,
        "swap": True
    },
    "QtCore.QRegularExpression": {
        "setPattern": True,
        "setPatternOptions": True,
        "swap": True
    },
    "QtCore.QRegularExpressionMatch": {
        "swap": True
    },
    "QtCore.QRegularExpressionMatchIterator": {
        "swap": True
    },
    "QtCore.QResource": {
        "setFileName": True,
        "setLocale": True
    },
    "QtCore.QRunnable": {
        "setAutoDelete": True
    },
    "QtCore.QSaveFile": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "fileTime": True,
        "findChild": True,
        "getChar": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "map": True,
        "moveToThread": True,
        "open": True,
        "peek": True,
        "property": True,
        "putChar": True,
        "read": True,
        "readData": True,
        "readLine": True,
        "readLineData": True,
        "receivers": True,
        "removeEventFilter": True,
        "resize": True,
        "seek": True,
        "setCurrentReadChannel": True,
        "setCurrentWriteChannel": True,
        "setDirectWriteFallback": True,
        "setErrorString": True,
        "setFileName": True,
        "setFileTime": True,
        "setObjectName": True,
        "setOpenMode": True,
        "setParent": True,
        "setPermissions": True,
        "setProperty": True,
        "setTextModeEnabled": True,
        "skip": True,
        "startTimer": True,
        "tr": True,
        "ungetChar": True,
        "unmap": True,
        "waitForBytesWritten": True,
        "waitForReadyRead": True,
        "write": True,
        "writeData": True
    },
    "QtCore.QSemaphore": {
        "acquire": True,
        "release": True
    },
    "QtCore.QSemaphoreReleaser": {
        "swap": True
    },
    "QtCore.QSequentialAnimationGroup": {
        "addAnimation": True,
        "addPause": True,
        "animationAt": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "indexOfAnimation": True,
        "inherits": True,
        "insertAnimation": True,
        "insertPause": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setDirection": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "start": True,
        "startTimer": True,
        "takeAnimation": True,
        "tr": True,
        "updateCurrentTime": True,
        "updateDirection": True,
        "updateState": True
    },
    "QtCore.QSettings": {
        "beginGroup": True,
        "beginReadArray": True,
        "beginWriteArray": True,
        "blockSignals": True,
        "connectNotify": True,
        "contains": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "remove": True,
        "removeEventFilter": True,
        "setArrayIndex": True,
        "setAtomicSyncRequired": True,
        "setFallbacksEnabled": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setValue": True,
        "startTimer": True,
        "tr": True,
        "value": True
    },
    "QtCore.QSignalMapper": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeMappings": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QSignalTransition": {
        "addAnimation": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeAnimation": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSenderObject": True,
        "setSignal": True,
        "setTargetState": True,
        "setTargetStates": True,
        "setTransitionType": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QSize": {
        "boundedTo": True,
        "expandedTo": True,
        "grownBy": True,
        "setHeight": True,
        "setWidth": True,
        "shrunkBy": True
    },
    "QtCore.QSizeF": {
        "boundedTo": True,
        "expandedTo": True,
        "grownBy": True,
        "setHeight": True,
        "setWidth": True,
        "shrunkBy": True
    },
    "QtCore.QSocketNotifier": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setEnabled": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QSortFilterProxyModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "filterAcceptsColumn": True,
        "filterAcceptsRow": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "lessThan": True,
        "mapFromSource": True,
        "mapSelectionFromSource": True,
        "mapSelectionToSource": True,
        "mapToSource": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setDynamicSortFilter": True,
        "setFilterCaseSensitivity": True,
        "setFilterFixedString": True,
        "setFilterKeyColumn": True,
        "setFilterRole": True,
        "setFilterWildcard": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRecursiveFilteringEnabled": True,
        "setSortCaseSensitivity": True,
        "setSortLocaleAware": True,
        "setSortRole": True,
        "setSourceModel": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QState": {
        "assignProperty": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeTransition": True,
        "setChildMode": True,
        "setErrorState": True,
        "setInitialState": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QStateMachine": {
        "addDefaultAnimation": True,
        "addState": True,
        "assignProperty": True,
        "blockSignals": True,
        "cancelDelayedEvent": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeDefaultAnimation": True,
        "removeEventFilter": True,
        "removeState": True,
        "removeTransition": True,
        "setAnimated": True,
        "setChildMode": True,
        "setErrorState": True,
        "setGlobalRestorePolicy": True,
        "setInitialState": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRunning": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QStorageInfo": {
        "setPath": True,
        "swap": True
    },
    "QtCore.QStringListModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setStringList": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QSystemSemaphore": {
        "release": True,
        "setKey": True
    },
    "QtCore.QTemporaryDir": {
        "filePath": True,
        "setAutoRemove": True
    },
    "QtCore.QTemporaryFile": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "fileTime": True,
        "findChild": True,
        "getChar": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "map": True,
        "moveToThread": True,
        "peek": True,
        "property": True,
        "putChar": True,
        "read": True,
        "readData": True,
        "readLine": True,
        "readLineData": True,
        "receivers": True,
        "removeEventFilter": True,
        "rename": True,
        "seek": True,
        "setAutoRemove": True,
        "setCurrentReadChannel": True,
        "setCurrentWriteChannel": True,
        "setErrorString": True,
        "setFileName": True,
        "setFileTemplate": True,
        "setFileTime": True,
        "setObjectName": True,
        "setOpenMode": True,
        "setParent": True,
        "setProperty": True,
        "setTextModeEnabled": True,
        "skip": True,
        "startTimer": True,
        "tr": True,
        "ungetChar": True,
        "unmap": True,
        "waitForBytesWritten": True,
        "waitForReadyRead": True,
        "write": True,
        "writeData": True
    },
    "QtCore.QTextBoundaryFinder": {
        "setPosition": True
    },
    "QtCore.QTextCodec": {
        "convertToUnicode": True,
        "fromUnicode": True,
        "makeDecoder": True,
        "makeEncoder": True
    },
    "QtCore.QTextDecoder": {
        "toUnicode": True
    },
    "QtCore.QTextEncoder": {
        "fromUnicode": True
    },
    "QtCore.QTextStream": {
        "read": True,
        "readLine": True,
        "seek": True,
        "setAutoDetectUnicode": True,
        "setDevice": True,
        "setFieldAlignment": True,
        "setFieldWidth": True,
        "setGenerateByteOrderMark": True,
        "setIntegerBase": True,
        "setLocale": True,
        "setNumberFlags": True,
        "setPadChar": True,
        "setRealNumberNotation": True,
        "setRealNumberPrecision": True,
        "setStatus": True
    },
    "QtCore.QThread": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "exit": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setEventDispatcher": True,
        "setObjectName": True,
        "setParent": True,
        "setPriority": True,
        "setProperty": True,
        "setStackSize": True,
        "start": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QThreadPool": {
        "blockSignals": True,
        "cancel": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setExpiryTimeout": True,
        "setMaxThreadCount": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setStackSize": True,
        "start": True,
        "startTimer": True,
        "tr": True,
        "tryStart": True,
        "tryTake": True,
        "waitForDone": True
    },
    "QtCore.QTime": {
        "addMSecs": True,
        "addSecs": True,
        "msecsTo": True,
        "secsTo": True,
        "setHMS": True
    },
    "QtCore.QTimeLine": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "frameForTime": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setCurveShape": True,
        "setDirection": True,
        "setDuration": True,
        "setEasingCurve": True,
        "setEndFrame": True,
        "setFrameRange": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "setStartFrame": True,
        "setUpdateInterval": True,
        "startTimer": True,
        "tr": True,
        "valueForTime": True
    },
    "QtCore.QTimeZone": {
        "abbreviation": True,
        "daylightTimeOffset": True,
        "isDaylightTime": True,
        "nextTransition": True,
        "offsetData": True,
        "offsetFromUtc": True,
        "previousTransition": True,
        "standardTimeOffset": True,
        "swap": True,
        "transitions": True
    },
    "QtCore.QTimer": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setInterval": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSingleShot": True,
        "setTimerType": True,
        "startTimer": True,
        "timerEvent": True,
        "tr": True
    },
    "QtCore.QTimerEvent": {
        "setAccepted": True
    },
    "QtCore.QTranslator": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True,
        "translate": True
    },
    "QtCore.QTransposeProxyModel": {
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "index": True,
        "inherits": True,
        "insertColumn": True,
        "insertColumns": True,
        "insertRow": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "itemData": True,
        "killTimer": True,
        "mapFromSource": True,
        "mapSelectionFromSource": True,
        "mapSelectionToSource": True,
        "mapToSource": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setData": True,
        "setHeaderData": True,
        "setItemData": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSourceModel": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QUrl": {
        "adjusted": True,
        "authority": True,
        "fileName": True,
        "fragment": True,
        "host": True,
        "isParentOf": True,
        "matches": True,
        "password": True,
        "path": True,
        "port": True,
        "query": True,
        "resolved": True,
        "setAuthority": True,
        "setFragment": True,
        "setHost": True,
        "setPassword": True,
        "setPath": True,
        "setPort": True,
        "setScheme": True,
        "setUrl": True,
        "setUserInfo": True,
        "setUserName": True,
        "swap": True,
        "toDisplayString": True,
        "toEncoded": True,
        "toString": True,
        "topLevelDomain": True,
        "url": True,
        "userInfo": True,
        "userName": True
    },
    "QtCore.QUrlQuery": {
        "addQueryItem": True,
        "allQueryItemValues": True,
        "hasQueryItem": True,
        "query": True,
        "queryItemValue": True,
        "queryItems": True,
        "removeAllQueryItems": True,
        "removeQueryItem": True,
        "setQuery": True,
        "setQueryDelimiters": True,
        "setQueryItems": True,
        "swap": True,
        "toString": True
    },
    "QtCore.QVariantAnimation": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "interpolated": True,
        "isSignalConnected": True,
        "keyValueAt": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCurrentTime": True,
        "setDirection": True,
        "setDuration": True,
        "setEasingCurve": True,
        "setEndValue": True,
        "setKeyValueAt": True,
        "setKeyValues": True,
        "setLoopCount": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "setStartValue": True,
        "start": True,
        "startTimer": True,
        "tr": True,
        "updateCurrentTime": True,
        "updateCurrentValue": True,
        "updateDirection": True,
        "updateState": True
    },
    "QtCore.QVersionNumber": {
        "isPrefixOf": True,
        "segmentAt": True
    },
    "QtCore.QWinEventNotifier": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setEnabled": True,
        "setHandle": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtCore.QXmlStreamAttributes": {
        "at": True,
        "contains": True,
        "fill": True,
        "indexOf": True,
        "lastIndexOf": True,
        "prepend": True,
        "replace": True
    },
    "QtCore.QXmlStreamEntityResolver": {
        "resolveUndeclaredEntity": True
    },
    "QtCore.QXmlStreamReader": {
        "addExtraNamespaceDeclaration": True,
        "addExtraNamespaceDeclarations": True,
        "raiseError": True,
        "readElementText": True,
        "setDevice": True,
        "setEntityExpansionLimit": True,
        "setEntityResolver": True,
        "setNamespaceProcessing": True
    },
    "QtCore.QXmlStreamWriter": {
        "setAutoFormatting": True,
        "setAutoFormattingIndent": True,
        "setDevice": True,
        "writeAttributes": True,
        "writeCDATA": True,
        "writeCharacters": True,
        "writeComment": True,
        "writeCurrentToken": True,
        "writeDTD": True,
        "writeDefaultNamespace": True,
        "writeEntityReference": True,
        "writeNamespace": True,
        "writeProcessingInstruction": True
    },
    "QtGui.QAbstractTextDocumentLayout": {
        "anchorAt": True,
        "blockBoundingRect": True,
        "blockSignals": True,
        "blockWithMarkerAt": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "documentChanged": True,
        "draw": True,
        "drawInlineObject": True,
        "findChild": True,
        "format": True,
        "formatAt": True,
        "frameBoundingRect": True,
        "handlerForObject": True,
        "hitTest": True,
        "imageAt": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "positionInlineObject": True,
        "property": True,
        "receivers": True,
        "registerHandler": True,
        "removeEventFilter": True,
        "resizeInlineObject": True,
        "setObjectName": True,
        "setPaintDevice": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True,
        "unregisterHandler": True
    },
    "QtGui.QActionEvent": {
        "setAccepted": True
    },
    "QtGui.QBackingStore": {
        "beginPaint": True,
        "flush": True,
        "resize": True,
        "scroll": True,
        "setStaticContents": True
    },
    "QtGui.QBitmap": {
        "convertFromImage": True,
        "createHeuristicMask": True,
        "createMaskFromColor": True,
        "load": True,
        "metric": True,
        "scaledToHeight": True,
        "scaledToWidth": True,
        "setDevicePixelRatio": True,
        "setMask": True
    },
    "QtGui.QBrush": {
        "setStyle": True,
        "setTexture": True,
        "setTextureImage": True,
        "setTransform": True,
        "swap": True
    },
    "QtGui.QClipboard": {
        "blockSignals": True,
        "clear": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "image": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "mimeData": True,
        "moveToThread": True,
        "pixmap": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setImage": True,
        "setMimeData": True,
        "setObjectName": True,
        "setParent": True,
        "setPixmap": True,
        "setProperty": True,
        "setText": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QCloseEvent": {
        "setAccepted": True
    },
    "QtGui.QColor": {
        "convertTo": True,
        "darker": True,
        "lighter": True,
        "setAlpha": True,
        "setAlphaF": True,
        "setBlue": True,
        "setBlueF": True,
        "setCmyk": True,
        "setCmykF": True,
        "setGreen": True,
        "setGreenF": True,
        "setHsl": True,
        "setHslF": True,
        "setHsv": True,
        "setHsvF": True,
        "setNamedColor": True,
        "setRed": True,
        "setRedF": True,
        "setRgbF": True,
        "setRgba": True
    },
    "QtGui.QColorSpace": {
        "setTransferFunction": True,
        "swap": True,
        "withTransferFunction": True
    },
    "QtGui.QConicalGradient": {
        "setAngle": True,
        "setColorAt": True,
        "setCoordinateMode": True,
        "setSpread": True,
        "setStops": True
    },
    "QtGui.QContextMenuEvent": {
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QCursor": {
        "setShape": True,
        "swap": True
    },
    "QtGui.QDoubleValidator": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setBottom": True,
        "setDecimals": True,
        "setLocale": True,
        "setNotation": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRange": True,
        "setTop": True,
        "startTimer": True,
        "tr": True,
        "validate": True
    },
    "QtGui.QDrag": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "dragCursor": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setDragCursor": True,
        "setHotSpot": True,
        "setMimeData": True,
        "setObjectName": True,
        "setParent": True,
        "setPixmap": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QDragEnterEvent": {
        "setAccepted": True,
        "setDropAction": True
    },
    "QtGui.QDragLeaveEvent": {
        "setAccepted": True
    },
    "QtGui.QDragMoveEvent": {
        "setAccepted": True,
        "setDropAction": True
    },
    "QtGui.QDropEvent": {
        "setAccepted": True,
        "setDropAction": True
    },
    "QtGui.QEnterEvent": {
        "setAccepted": True
    },
    "QtGui.QExposeEvent": {
        "setAccepted": True
    },
    "QtGui.QFileOpenEvent": {
        "openFile": True,
        "setAccepted": True
    },
    "QtGui.QFocusEvent": {
        "setAccepted": True
    },
    "QtGui.QFont": {
        "fromString": True,
        "isCopyOf": True,
        "setBold": True,
        "setCapitalization": True,
        "setFamilies": True,
        "setFamily": True,
        "setFixedPitch": True,
        "setHintingPreference": True,
        "setItalic": True,
        "setKerning": True,
        "setLetterSpacing": True,
        "setOverline": True,
        "setPixelSize": True,
        "setPointSize": True,
        "setPointSizeF": True,
        "setRawMode": True,
        "setRawName": True,
        "setStretch": True,
        "setStrikeOut": True,
        "setStyle": True,
        "setStyleHint": True,
        "setStyleName": True,
        "setStyleStrategy": True,
        "setUnderline": True,
        "setWeight": True,
        "setWordSpacing": True,
        "swap": True
    },
    "QtGui.QFontDatabase": {
        "bold": True,
        "families": True,
        "font": True,
        "isBitmapScalable": True,
        "isFixedPitch": True,
        "isPrivateFamily": True,
        "isScalable": True,
        "isSmoothlyScalable": True,
        "italic": True,
        "pointSizes": True,
        "smoothSizes": True,
        "styles": True,
        "weight": True
    },
    "QtGui.QFontInfo": {
        "swap": True
    },
    "QtGui.QFontMetrics": {
        "boundingRectChar": True,
        "elidedText": True,
        "inFont": True,
        "inFontUcs4": True,
        "leftBearing": True,
        "rightBearing": True,
        "size": True,
        "swap": True,
        "tightBoundingRect": True,
        "widthChar": True
    },
    "QtGui.QFontMetricsF": {
        "boundingRectChar": True,
        "elidedText": True,
        "inFont": True,
        "inFontUcs4": True,
        "leftBearing": True,
        "rightBearing": True,
        "size": True,
        "swap": True,
        "tightBoundingRect": True,
        "width": True,
        "widthChar": True
    },
    "QtGui.QGradient": {
        "setColorAt": True,
        "setCoordinateMode": True,
        "setSpread": True,
        "setStops": True
    },
    "QtGui.QGuiApplication": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "event": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "installNativeEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "notify": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeNativeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QHelpEvent": {
        "setAccepted": True
    },
    "QtGui.QHideEvent": {
        "setAccepted": True
    },
    "QtGui.QHoverEvent": {
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QIcon": {
        "addFile": True,
        "addPixmap": True,
        "availableSizes": True,
        "setIsMask": True,
        "swap": True
    },
    "QtGui.QIconDragEvent": {
        "setAccepted": True
    },
    "QtGui.QIconEngine": {
        "actualSize": True,
        "addFile": True,
        "addPixmap": True,
        "availableSizes": True,
        "paint": True,
        "pixmap": True,
        "read": True,
        "scaledPixmap": True,
        "write": True
    },
    "QtGui.QImage": {
        "color": True,
        "constScanLine": True,
        "convertTo": True,
        "convertToColorSpace": True,
        "convertedToColorSpace": True,
        "createAlphaMask": True,
        "createHeuristicMask": True,
        "createMaskFromColor": True,
        "invertPixels": True,
        "loadFromData": True,
        "metric": True,
        "mirrored": True,
        "reinterpretAsFormat": True,
        "scaledToHeight": True,
        "scaledToWidth": True,
        "scanLine": True,
        "setColor": True,
        "setColorCount": True,
        "setColorSpace": True,
        "setColorTable": True,
        "setDevicePixelRatio": True,
        "setDotsPerMeterX": True,
        "setDotsPerMeterY": True,
        "setOffset": True,
        "setText": True,
        "smoothScaled": True,
        "swap": True,
        "text": True
    },
    "QtGui.QImageIOHandler": {
        "jumpToImage": True,
        "option": True,
        "read": True,
        "setDevice": True,
        "setFormat": True,
        "setOption": True,
        "supportsOption": True,
        "write": True
    },
    "QtGui.QImageReader": {
        "jumpToImage": True,
        "setAutoDetectImageFormat": True,
        "setAutoTransform": True,
        "setBackgroundColor": True,
        "setClipRect": True,
        "setDecideFormatFromContent": True,
        "setDevice": True,
        "setFileName": True,
        "setFormat": True,
        "setGamma": True,
        "setQuality": True,
        "setScaledClipRect": True,
        "setScaledSize": True,
        "supportsOption": True,
        "text": True
    },
    "QtGui.QImageWriter": {
        "setCompression": True,
        "setDevice": True,
        "setFileName": True,
        "setFormat": True,
        "setGamma": True,
        "setOptimizedWrite": True,
        "setProgressiveScanWrite": True,
        "setQuality": True,
        "setSubType": True,
        "setText": True,
        "setTransformation": True,
        "supportsOption": True,
        "write": True
    },
    "QtGui.QInputEvent": {
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QInputMethod": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "invokeAction": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setInputItemRectangle": True,
        "setInputItemTransform": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setVisible": True,
        "startTimer": True,
        "tr": True,
        "update": True
    },
    "QtGui.QInputMethodEvent": {
        "setAccepted": True,
        "setCommitString": True
    },
    "QtGui.QInputMethodQueryEvent": {
        "setAccepted": True,
        "setValue": True,
        "value": True
    },
    "QtGui.QIntValidator": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setBottom": True,
        "setLocale": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRange": True,
        "setTop": True,
        "startTimer": True,
        "tr": True,
        "validate": True
    },
    "QtGui.QKeyEvent": {
        "matches": True,
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QKeySequence": {
        "matches": True,
        "swap": True,
        "toString": True
    },
    "QtGui.QLinearGradient": {
        "setColorAt": True,
        "setCoordinateMode": True,
        "setSpread": True,
        "setStops": True
    },
    "QtGui.QMatrix2x2": {
        "fill": True
    },
    "QtGui.QMatrix2x3": {
        "fill": True
    },
    "QtGui.QMatrix2x4": {
        "fill": True
    },
    "QtGui.QMatrix3x2": {
        "fill": True
    },
    "QtGui.QMatrix3x3": {
        "fill": True
    },
    "QtGui.QMatrix3x4": {
        "fill": True
    },
    "QtGui.QMatrix4x2": {
        "fill": True
    },
    "QtGui.QMatrix4x3": {
        "fill": True
    },
    "QtGui.QMatrix4x4": {
        "column": True,
        "fill": True,
        "frustum": True,
        "lookAt": True,
        "mapVector": True,
        "perspective": True,
        "row": True,
        "setColumn": True,
        "setRow": True
    },
    "QtGui.QMouseEvent": {
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QMoveEvent": {
        "setAccepted": True
    },
    "QtGui.QMovie": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "jumpToFrame": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setBackgroundColor": True,
        "setCacheMode": True,
        "setDevice": True,
        "setFileName": True,
        "setFormat": True,
        "setObjectName": True,
        "setParent": True,
        "setPaused": True,
        "setProperty": True,
        "setScaledSize": True,
        "setSpeed": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QNativeGestureEvent": {
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QOffscreenSurface": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setFormat": True,
        "setNativeHandle": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setScreen": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLBuffer": {
        "map": True,
        "mapRange": True,
        "read": True,
        "setUsagePattern": True,
        "write": True
    },
    "QtGui.QOpenGLContext": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "hasExtension": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "makeCurrent": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setFormat": True,
        "setNativeHandle": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setScreen": True,
        "setShareContext": True,
        "startTimer": True,
        "swapBuffers": True,
        "tr": True,
        "versionFunctions": True
    },
    "QtGui.QOpenGLContextGroup": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLDebugLogger": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "logMessage": True,
        "moveToThread": True,
        "property": True,
        "pushGroup": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startLogging": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLDebugMessage": {
        "swap": True
    },
    "QtGui.QOpenGLFramebufferObject": {
        "setAttachment": True
    },
    "QtGui.QOpenGLFramebufferObjectFormat": {
        "setAttachment": True,
        "setInternalTextureFormat": True,
        "setMipmap": True,
        "setSamples": True,
        "setTextureTarget": True
    },
    "QtGui.QOpenGLPixelTransferOptions": {
        "setAlignment": True,
        "setImageHeight": True,
        "setLeastSignificantByteFirst": True,
        "setRowLength": True,
        "setSkipImages": True,
        "setSkipPixels": True,
        "setSkipRows": True,
        "setSwapBytesEnabled": True,
        "swap": True
    },
    "QtGui.QOpenGLShader": {
        "blockSignals": True,
        "compileSourceFile": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLShaderProgram": {
        "addCacheableShaderFromSourceFile": True,
        "addShader": True,
        "addShaderFromSourceFile": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "removeShader": True,
        "setDefaultInnerTessellationLevels": True,
        "setDefaultOuterTessellationLevels": True,
        "setObjectName": True,
        "setParent": True,
        "setPatchVertexCount": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLTexture": {
        "createTextureView": True,
        "setAutoMipMapGenerationEnabled": True,
        "setComparisonFunction": True,
        "setComparisonMode": True,
        "setDepthStencilMode": True,
        "setFixedSamplePositions": True,
        "setFormat": True,
        "setLayers": True,
        "setLevelOfDetailRange": True,
        "setLevelofDetailBias": True,
        "setMagnificationFilter": True,
        "setMaximumAnisotropy": True,
        "setMaximumLevelOfDetail": True,
        "setMinMagFilters": True,
        "setMinificationFilter": True,
        "setMinimumLevelOfDetail": True,
        "setMipBaseLevel": True,
        "setMipLevelRange": True,
        "setMipLevels": True,
        "setMipMaxLevel": True,
        "setSamples": True,
        "setSize": True,
        "swizzleMask": True,
        "wrapMode": True
    },
    "QtGui.QOpenGLTextureBlitter": {
        "bind": True,
        "setOpacity": True,
        "setRedBlueSwizzle": True
    },
    "QtGui.QOpenGLTimeMonitor": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setSampleCount": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLTimerQuery": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLVersionProfile": {
        "setProfile": True,
        "setVersion": True
    },
    "QtGui.QOpenGLVertexArrayObject": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QOpenGLWindow": {
        "alert": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "exposeEvent": True,
        "findChild": True,
        "focusInEvent": True,
        "focusOutEvent": True,
        "hideEvent": True,
        "inherits": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isSignalConnected": True,
        "keyPressEvent": True,
        "keyReleaseEvent": True,
        "killTimer": True,
        "mapFromGlobal": True,
        "mapToGlobal": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveEvent": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "reportContentOrientationChange": True,
        "resizeGL": True,
        "setBaseSize": True,
        "setCursor": True,
        "setFilePath": True,
        "setFlag": True,
        "setFlags": True,
        "setFormat": True,
        "setFramePosition": True,
        "setHeight": True,
        "setIcon": True,
        "setKeyboardGrabEnabled": True,
        "setMask": True,
        "setMaximumHeight": True,
        "setMaximumSize": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumSize": True,
        "setMinimumWidth": True,
        "setModality": True,
        "setMouseGrabEnabled": True,
        "setObjectName": True,
        "setOpacity": True,
        "setProperty": True,
        "setScreen": True,
        "setSizeIncrement": True,
        "setSurfaceType": True,
        "setTitle": True,
        "setTransientParent": True,
        "setVisibility": True,
        "setVisible": True,
        "setWidth": True,
        "setWindowState": True,
        "setWindowStates": True,
        "setX": True,
        "setY": True,
        "showEvent": True,
        "startSystemResize": True,
        "startTimer": True,
        "tabletEvent": True,
        "touchEvent": True,
        "tr": True,
        "wheelEvent": True
    },
    "QtGui.QPageLayout": {
        "fullRectPixels": True,
        "isEquivalentTo": True,
        "marginsPixels": True,
        "paintRectPixels": True,
        "setBottomMargin": True,
        "setLeftMargin": True,
        "setMargins": True,
        "setMinimumMargins": True,
        "setMode": True,
        "setOrientation": True,
        "setPageSize": True,
        "setRightMargin": True,
        "setTopMargin": True,
        "setUnits": True,
        "swap": True
    },
    "QtGui.QPageSize": {
        "isEquivalentTo": True,
        "rect": True,
        "rectPixels": True,
        "swap": True
    },
    "QtGui.QPagedPaintDevice": {
        "metric": True,
        "setMargins": True,
        "setPageLayout": True,
        "setPageOrientation": True,
        "setPageSizeMM": True
    },
    "QtGui.QPaintDevice": {
        "metric": True
    },
    "QtGui.QPaintDeviceWindow": {
        "alert": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "exposeEvent": True,
        "findChild": True,
        "focusInEvent": True,
        "focusOutEvent": True,
        "hideEvent": True,
        "inherits": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isSignalConnected": True,
        "keyPressEvent": True,
        "keyReleaseEvent": True,
        "killTimer": True,
        "mapFromGlobal": True,
        "mapToGlobal": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveEvent": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "reportContentOrientationChange": True,
        "resizeEvent": True,
        "setBaseSize": True,
        "setCursor": True,
        "setFilePath": True,
        "setFlag": True,
        "setFlags": True,
        "setFormat": True,
        "setFramePosition": True,
        "setHeight": True,
        "setIcon": True,
        "setKeyboardGrabEnabled": True,
        "setMask": True,
        "setMaximumHeight": True,
        "setMaximumSize": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumSize": True,
        "setMinimumWidth": True,
        "setModality": True,
        "setMouseGrabEnabled": True,
        "setObjectName": True,
        "setOpacity": True,
        "setProperty": True,
        "setScreen": True,
        "setSizeIncrement": True,
        "setSurfaceType": True,
        "setTitle": True,
        "setTransientParent": True,
        "setVisibility": True,
        "setVisible": True,
        "setWidth": True,
        "setWindowState": True,
        "setWindowStates": True,
        "setX": True,
        "setY": True,
        "showEvent": True,
        "startSystemResize": True,
        "startTimer": True,
        "tabletEvent": True,
        "touchEvent": True,
        "tr": True,
        "wheelEvent": True
    },
    "QtGui.QPaintEngine": {
        "begin": True,
        "drawImage": True,
        "drawPath": True,
        "drawPixmap": True,
        "drawTextItem": True,
        "drawTiledPixmap": True,
        "hasFeature": True,
        "setActive": True,
        "updateState": True
    },
    "QtGui.QPaintEvent": {
        "setAccepted": True
    },
    "QtGui.QPainter": {
        "begin": True,
        "drawPath": True,
        "drawPixmapFragments": True,
        "fillPath": True,
        "rotate": True,
        "scale": True,
        "setBackground": True,
        "setBackgroundMode": True,
        "setClipPath": True,
        "setClipRegion": True,
        "setClipping": True,
        "setCompositionMode": True,
        "setFont": True,
        "setLayoutDirection": True,
        "setOpacity": True,
        "setRenderHint": True,
        "setRenderHints": True,
        "setTransform": True,
        "setViewTransformEnabled": True,
        "setWorldMatrixEnabled": True,
        "setWorldTransform": True,
        "shear": True,
        "strokePath": True,
        "testRenderHint": True
    },
    "QtGui.QPainterPath": {
        "addPath": True,
        "addPolygon": True,
        "addRegion": True,
        "angleAtPercent": True,
        "connectPath": True,
        "elementAt": True,
        "intersected": True,
        "percentAtLength": True,
        "pointAtPercent": True,
        "reserve": True,
        "setElementPositionAt": True,
        "setFillRule": True,
        "slopeAtPercent": True,
        "subtracted": True,
        "swap": True,
        "united": True
    },
    "QtGui.QPainterPathStroker": {
        "createStroke": True,
        "setCapStyle": True,
        "setCurveThreshold": True,
        "setDashOffset": True,
        "setJoinStyle": True,
        "setMiterLimit": True,
        "setWidth": True
    },
    "QtGui.QPalette": {
        "isBrushSet": True,
        "isCopyOf": True,
        "isEqual": True,
        "setColorGroup": True,
        "setCurrentColorGroup": True,
        "swap": True
    },
    "QtGui.QPdfWriter": {
        "addFileAttachment": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "metric": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setCreator": True,
        "setDocumentXmpMetadata": True,
        "setMargins": True,
        "setObjectName": True,
        "setPageLayout": True,
        "setPageOrientation": True,
        "setPageSize": True,
        "setPageSizeMM": True,
        "setParent": True,
        "setPdfVersion": True,
        "setProperty": True,
        "setResolution": True,
        "setTitle": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QPen": {
        "setBrush": True,
        "setCapStyle": True,
        "setColor": True,
        "setCosmetic": True,
        "setDashOffset": True,
        "setDashPattern": True,
        "setJoinStyle": True,
        "setMiterLimit": True,
        "setStyle": True,
        "setWidth": True,
        "setWidthF": True,
        "swap": True
    },
    "QtGui.QPicture": {
        "metric": True,
        "play": True,
        "setBoundingRect": True,
        "setData": True,
        "swap": True
    },
    "QtGui.QPictureIO": {
        "setDescription": True,
        "setFileName": True,
        "setFormat": True,
        "setGamma": True,
        "setIODevice": True,
        "setParameters": True,
        "setPicture": True,
        "setQuality": True,
        "setStatus": True
    },
    "QtGui.QPixmap": {
        "convertFromImage": True,
        "createHeuristicMask": True,
        "createMaskFromColor": True,
        "load": True,
        "metric": True,
        "scaledToHeight": True,
        "scaledToWidth": True,
        "setDevicePixelRatio": True,
        "setMask": True,
        "swap": True
    },
    "QtGui.QPolygon": {
        "at": True,
        "contains": True,
        "containsPoint": True,
        "fill": True,
        "indexOf": True,
        "intersected": True,
        "intersects": True,
        "lastIndexOf": True,
        "mid": True,
        "prepend": True,
        "replace": True,
        "subtracted": True,
        "swap": True,
        "united": True
    },
    "QtGui.QPolygonF": {
        "at": True,
        "contains": True,
        "containsPoint": True,
        "fill": True,
        "indexOf": True,
        "intersected": True,
        "intersects": True,
        "lastIndexOf": True,
        "mid": True,
        "prepend": True,
        "replace": True,
        "subtracted": True,
        "swap": True,
        "united": True
    },
    "QtGui.QQuaternion": {
        "getAxes": True,
        "rotatedVector": True,
        "setScalar": True,
        "setX": True,
        "setY": True,
        "setZ": True
    },
    "QtGui.QRadialGradient": {
        "setCenterRadius": True,
        "setColorAt": True,
        "setCoordinateMode": True,
        "setFocalRadius": True,
        "setRadius": True,
        "setSpread": True,
        "setStops": True
    },
    "QtGui.QRasterWindow": {
        "alert": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "exposeEvent": True,
        "findChild": True,
        "focusInEvent": True,
        "focusOutEvent": True,
        "hideEvent": True,
        "inherits": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isSignalConnected": True,
        "keyPressEvent": True,
        "keyReleaseEvent": True,
        "killTimer": True,
        "mapFromGlobal": True,
        "mapToGlobal": True,
        "metric": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveEvent": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "reportContentOrientationChange": True,
        "resizeEvent": True,
        "setBaseSize": True,
        "setCursor": True,
        "setFilePath": True,
        "setFlag": True,
        "setFlags": True,
        "setFormat": True,
        "setFramePosition": True,
        "setHeight": True,
        "setIcon": True,
        "setKeyboardGrabEnabled": True,
        "setMask": True,
        "setMaximumHeight": True,
        "setMaximumSize": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumSize": True,
        "setMinimumWidth": True,
        "setModality": True,
        "setMouseGrabEnabled": True,
        "setObjectName": True,
        "setOpacity": True,
        "setProperty": True,
        "setScreen": True,
        "setSizeIncrement": True,
        "setSurfaceType": True,
        "setTitle": True,
        "setTransientParent": True,
        "setVisibility": True,
        "setVisible": True,
        "setWidth": True,
        "setWindowState": True,
        "setWindowStates": True,
        "setX": True,
        "setY": True,
        "showEvent": True,
        "startSystemResize": True,
        "startTimer": True,
        "tabletEvent": True,
        "touchEvent": True,
        "tr": True,
        "wheelEvent": True
    },
    "QtGui.QRawFont": {
        "alphaMapForGlyph": True,
        "boundingRect": True,
        "fontTable": True,
        "glyphIndexesForString": True,
        "loadFromData": True,
        "loadFromFile": True,
        "pathForGlyph": True,
        "setPixelSize": True,
        "swap": True
    },
    "QtGui.QRegExpValidator": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setLocale": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRegExp": True,
        "startTimer": True,
        "tr": True,
        "validate": True
    },
    "QtGui.QRegion": {
        "setRects": True,
        "subtracted": True,
        "swap": True,
        "xored": True
    },
    "QtGui.QRegularExpressionValidator": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setLocale": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRegularExpression": True,
        "startTimer": True,
        "tr": True,
        "validate": True
    },
    "QtGui.QResizeEvent": {
        "setAccepted": True
    },
    "QtGui.QScreen": {
        "angleBetween": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "grabWindow": True,
        "inherits": True,
        "installEventFilter": True,
        "isLandscape": True,
        "isPortrait": True,
        "isSignalConnected": True,
        "killTimer": True,
        "mapBetween": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setOrientationUpdateMask": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True,
        "transformBetween": True,
        "virtualSiblingAt": True
    },
    "QtGui.QScrollEvent": {
        "setAccepted": True
    },
    "QtGui.QScrollPrepareEvent": {
        "setAccepted": True,
        "setContentPos": True,
        "setContentPosRange": True,
        "setViewportSize": True
    },
    "QtGui.QSessionManager": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setDiscardCommand": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRestartCommand": True,
        "setRestartHint": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QShortcutEvent": {
        "setAccepted": True
    },
    "QtGui.QShowEvent": {
        "setAccepted": True
    },
    "QtGui.QStandardItem": {
        "appendColumn": True,
        "appendRows": True,
        "child": True,
        "data": True,
        "insertColumn": True,
        "insertColumns": True,
        "read": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeRow": True,
        "removeRows": True,
        "setAccessibleDescription": True,
        "setAccessibleText": True,
        "setAutoTristate": True,
        "setBackground": True,
        "setCheckState": True,
        "setCheckable": True,
        "setColumnCount": True,
        "setData": True,
        "setDragEnabled": True,
        "setDropEnabled": True,
        "setEditable": True,
        "setEnabled": True,
        "setFlags": True,
        "setFont": True,
        "setForeground": True,
        "setIcon": True,
        "setRowCount": True,
        "setSelectable": True,
        "setSizeHint": True,
        "setStatusTip": True,
        "setText": True,
        "setTextAlignment": True,
        "setToolTip": True,
        "setTristate": True,
        "setUserTristate": True,
        "setWhatsThis": True,
        "sortChildren": True,
        "takeChild": True,
        "takeColumn": True,
        "takeRow": True,
        "write": True
    },
    "QtGui.QStandardItemModel": {
        "appendColumn": True,
        "beginInsertColumns": True,
        "beginInsertRows": True,
        "beginMoveColumns": True,
        "beginMoveRows": True,
        "beginRemoveColumns": True,
        "beginRemoveRows": True,
        "blockSignals": True,
        "buddy": True,
        "canDropMimeData": True,
        "canFetchMore": True,
        "changePersistentIndex": True,
        "changePersistentIndexList": True,
        "checkIndex": True,
        "clearItemData": True,
        "columnCount": True,
        "connectNotify": True,
        "data": True,
        "decodeData": True,
        "disconnectNotify": True,
        "dropMimeData": True,
        "encodeData": True,
        "fetchMore": True,
        "findChild": True,
        "findItems": True,
        "flags": True,
        "hasChildren": True,
        "hasIndex": True,
        "headerData": True,
        "horizontalHeaderItem": True,
        "index": True,
        "indexFromItem": True,
        "inherits": True,
        "insertColumns": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "item": True,
        "itemData": True,
        "itemFromIndex": True,
        "killTimer": True,
        "match": True,
        "mimeData": True,
        "moveColumn": True,
        "moveColumns": True,
        "moveRow": True,
        "moveRows": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumn": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRow": True,
        "removeRows": True,
        "rowCount": True,
        "setColumnCount": True,
        "setData": True,
        "setHeaderData": True,
        "setHorizontalHeaderItem": True,
        "setHorizontalHeaderLabels": True,
        "setItemData": True,
        "setItemPrototype": True,
        "setItemRoleNames": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setRowCount": True,
        "setSortRole": True,
        "setVerticalHeaderItem": True,
        "setVerticalHeaderLabels": True,
        "sibling": True,
        "sort": True,
        "span": True,
        "startTimer": True,
        "takeColumn": True,
        "takeHorizontalHeaderItem": True,
        "takeItem": True,
        "takeRow": True,
        "takeVerticalHeaderItem": True,
        "tr": True,
        "verticalHeaderItem": True
    },
    "QtGui.QStaticText": {
        "prepare": True,
        "setPerformanceHint": True,
        "setText": True,
        "setTextFormat": True,
        "setTextOption": True,
        "setTextWidth": True,
        "swap": True
    },
    "QtGui.QStatusTipEvent": {
        "setAccepted": True
    },
    "QtGui.QStyleHints": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "setShowShortcutsInContextMenus": True,
        "setUseHoverEffects": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QSurfaceFormat": {
        "setAlphaBufferSize": True,
        "setBlueBufferSize": True,
        "setColorSpace": True,
        "setDepthBufferSize": True,
        "setGreenBufferSize": True,
        "setMajorVersion": True,
        "setMinorVersion": True,
        "setOptions": True,
        "setProfile": True,
        "setRedBufferSize": True,
        "setRenderableType": True,
        "setSamples": True,
        "setStencilBufferSize": True,
        "setStereo": True,
        "setSwapBehavior": True,
        "setSwapInterval": True,
        "setVersion": True
    },
    "QtGui.QSyntaxHighlighter": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "format": True,
        "highlightBlock": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "rehighlightBlock": True,
        "removeEventFilter": True,
        "setCurrentBlockState": True,
        "setCurrentBlockUserData": True,
        "setDocument": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QTabletEvent": {
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QTextBlock": {
        "contains": True,
        "setLineCount": True,
        "setRevision": True,
        "setUserData": True,
        "setUserState": True,
        "setVisible": True
    },
    "QtGui.QTextBlockFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setAlignment": True,
        "setBackground": True,
        "setBottomMargin": True,
        "setForeground": True,
        "setHeadingLevel": True,
        "setIndent": True,
        "setLayoutDirection": True,
        "setLeftMargin": True,
        "setLineHeight": True,
        "setMarker": True,
        "setNonBreakableLines": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "setPageBreakPolicy": True,
        "setRightMargin": True,
        "setTabPositions": True,
        "setTextIndent": True,
        "setTopMargin": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTextBlockGroup": {
        "blockFormatChanged": True,
        "blockInserted": True,
        "blockRemoved": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setFormat": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QTextCharFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setAnchor": True,
        "setAnchorHref": True,
        "setAnchorNames": True,
        "setBackground": True,
        "setFontCapitalization": True,
        "setFontFamilies": True,
        "setFontFamily": True,
        "setFontFixedPitch": True,
        "setFontHintingPreference": True,
        "setFontItalic": True,
        "setFontKerning": True,
        "setFontLetterSpacing": True,
        "setFontLetterSpacingType": True,
        "setFontOverline": True,
        "setFontPointSize": True,
        "setFontStretch": True,
        "setFontStrikeOut": True,
        "setFontStyleHint": True,
        "setFontStyleName": True,
        "setFontStyleStrategy": True,
        "setFontUnderline": True,
        "setFontWeight": True,
        "setFontWordSpacing": True,
        "setForeground": True,
        "setLayoutDirection": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "setTableCellColumnSpan": True,
        "setTableCellRowSpan": True,
        "setTextOutline": True,
        "setToolTip": True,
        "setUnderlineColor": True,
        "setUnderlineStyle": True,
        "setVerticalAlignment": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTextCursor": {
        "insertFragment": True,
        "insertFrame": True,
        "insertHtml": True,
        "isCopyOf": True,
        "mergeBlockCharFormat": True,
        "mergeBlockFormat": True,
        "mergeCharFormat": True,
        "movePosition": True,
        "select": True,
        "setBlockCharFormat": True,
        "setBlockFormat": True,
        "setCharFormat": True,
        "setKeepPositionOnInsert": True,
        "setPosition": True,
        "setVerticalMovementX": True,
        "setVisualNavigation": True,
        "swap": True
    },
    "QtGui.QTextDocument": {
        "addResource": True,
        "blockSignals": True,
        "characterAt": True,
        "clearUndoRedoStacks": True,
        "clone": True,
        "connectNotify": True,
        "createObject": True,
        "disconnectNotify": True,
        "drawContents": True,
        "findBlock": True,
        "findBlockByLineNumber": True,
        "findBlockByNumber": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "loadResource": True,
        "markContentsDirty": True,
        "metaInformation": True,
        "moveToThread": True,
        "object": True,
        "objectForFormat": True,
        "print_": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "resource": True,
        "setBaseUrl": True,
        "setDefaultCursorMoveStyle": True,
        "setDefaultFont": True,
        "setDefaultStyleSheet": True,
        "setDefaultTextOption": True,
        "setDocumentLayout": True,
        "setDocumentMargin": True,
        "setHtml": True,
        "setIndentWidth": True,
        "setMarkdown": True,
        "setMaximumBlockCount": True,
        "setMetaInformation": True,
        "setModified": True,
        "setObjectName": True,
        "setPageSize": True,
        "setParent": True,
        "setPlainText": True,
        "setProperty": True,
        "setTextWidth": True,
        "setUndoRedoEnabled": True,
        "setUseDesignMetrics": True,
        "startTimer": True,
        "toHtml": True,
        "toMarkdown": True,
        "tr": True
    },
    "QtGui.QTextDocumentFragment": {
        "toHtml": True
    },
    "QtGui.QTextDocumentWriter": {
        "setCodec": True,
        "setDevice": True,
        "setFileName": True,
        "setFormat": True
    },
    "QtGui.QTextFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setBackground": True,
        "setForeground": True,
        "setLayoutDirection": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTextFragment": {
        "contains": True
    },
    "QtGui.QTextFrame": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setFormat": True,
        "setFrameFormat": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QTextFrameFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setBackground": True,
        "setBorder": True,
        "setBorderBrush": True,
        "setBorderStyle": True,
        "setBottomMargin": True,
        "setForeground": True,
        "setLayoutDirection": True,
        "setLeftMargin": True,
        "setMargin": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "setPadding": True,
        "setPageBreakPolicy": True,
        "setPosition": True,
        "setRightMargin": True,
        "setTopMargin": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTextImageFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setAnchor": True,
        "setAnchorHref": True,
        "setAnchorNames": True,
        "setBackground": True,
        "setFontCapitalization": True,
        "setFontFamilies": True,
        "setFontFamily": True,
        "setFontFixedPitch": True,
        "setFontHintingPreference": True,
        "setFontItalic": True,
        "setFontKerning": True,
        "setFontLetterSpacing": True,
        "setFontLetterSpacingType": True,
        "setFontOverline": True,
        "setFontPointSize": True,
        "setFontStretch": True,
        "setFontStrikeOut": True,
        "setFontStyleHint": True,
        "setFontStyleName": True,
        "setFontStyleStrategy": True,
        "setFontUnderline": True,
        "setFontWeight": True,
        "setFontWordSpacing": True,
        "setForeground": True,
        "setHeight": True,
        "setLayoutDirection": True,
        "setName": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "setQuality": True,
        "setTableCellColumnSpan": True,
        "setTableCellRowSpan": True,
        "setTextOutline": True,
        "setToolTip": True,
        "setUnderlineColor": True,
        "setUnderlineStyle": True,
        "setVerticalAlignment": True,
        "setWidth": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTextInlineObject": {
        "setAscent": True,
        "setDescent": True,
        "setWidth": True
    },
    "QtGui.QTextLayout": {
        "draw": True,
        "isValidCursorPosition": True,
        "leftCursorPosition": True,
        "lineAt": True,
        "lineForTextPosition": True,
        "nextCursorPosition": True,
        "previousCursorPosition": True,
        "rightCursorPosition": True,
        "setAdditionalFormats": True,
        "setCacheEnabled": True,
        "setCursorMoveStyle": True,
        "setFont": True,
        "setFormats": True,
        "setPosition": True,
        "setPreeditArea": True,
        "setText": True,
        "setTextOption": True
    },
    "QtGui.QTextLength": {
        "value": True
    },
    "QtGui.QTextLine": {
        "cursorToX": True,
        "draw": True,
        "setLeadingIncluded": True,
        "setLineWidth": True,
        "setPosition": True,
        "xToCursor": True
    },
    "QtGui.QTextList": {
        "add": True,
        "blockFormatChanged": True,
        "blockInserted": True,
        "blockRemoved": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "item": True,
        "itemNumber": True,
        "itemText": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "remove": True,
        "removeEventFilter": True,
        "removeItem": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QTextListFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setBackground": True,
        "setForeground": True,
        "setIndent": True,
        "setLayoutDirection": True,
        "setNumberPrefix": True,
        "setNumberSuffix": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "setStyle": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTextObject": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setFormat": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QTextObjectInterface": {
        "drawObject": True,
        "intrinsicSize": True
    },
    "QtGui.QTextOption": {
        "setAlignment": True,
        "setFlags": True,
        "setTabArray": True,
        "setTabStop": True,
        "setTabStopDistance": True,
        "setTabs": True,
        "setTextDirection": True,
        "setUseDesignMetrics": True,
        "setWrapMode": True
    },
    "QtGui.QTextTable": {
        "appendColumns": True,
        "appendRows": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "inherits": True,
        "insertColumns": True,
        "insertRows": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeColumns": True,
        "removeEventFilter": True,
        "removeRows": True,
        "resize": True,
        "rowEnd": True,
        "rowStart": True,
        "setFrameFormat": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "splitCell": True,
        "startTimer": True,
        "tr": True
    },
    "QtGui.QTextTableCell": {
        "setFormat": True
    },
    "QtGui.QTextTableCellFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setAnchor": True,
        "setAnchorHref": True,
        "setAnchorNames": True,
        "setBackground": True,
        "setBorder": True,
        "setBorderBrush": True,
        "setBorderStyle": True,
        "setBottomBorder": True,
        "setBottomBorderBrush": True,
        "setBottomBorderStyle": True,
        "setBottomPadding": True,
        "setFontCapitalization": True,
        "setFontFamilies": True,
        "setFontFamily": True,
        "setFontFixedPitch": True,
        "setFontHintingPreference": True,
        "setFontItalic": True,
        "setFontKerning": True,
        "setFontLetterSpacing": True,
        "setFontLetterSpacingType": True,
        "setFontOverline": True,
        "setFontPointSize": True,
        "setFontStretch": True,
        "setFontStrikeOut": True,
        "setFontStyleHint": True,
        "setFontStyleName": True,
        "setFontStyleStrategy": True,
        "setFontUnderline": True,
        "setFontWeight": True,
        "setFontWordSpacing": True,
        "setForeground": True,
        "setLayoutDirection": True,
        "setLeftBorder": True,
        "setLeftBorderBrush": True,
        "setLeftBorderStyle": True,
        "setLeftPadding": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "setPadding": True,
        "setRightBorder": True,
        "setRightBorderBrush": True,
        "setRightBorderStyle": True,
        "setRightPadding": True,
        "setTableCellColumnSpan": True,
        "setTableCellRowSpan": True,
        "setTextOutline": True,
        "setToolTip": True,
        "setTopBorder": True,
        "setTopBorderBrush": True,
        "setTopBorderStyle": True,
        "setTopPadding": True,
        "setUnderlineColor": True,
        "setUnderlineStyle": True,
        "setVerticalAlignment": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTextTableFormat": {
        "boolProperty": True,
        "brushProperty": True,
        "clearProperty": True,
        "colorProperty": True,
        "doubleProperty": True,
        "hasProperty": True,
        "intProperty": True,
        "lengthProperty": True,
        "lengthVectorProperty": True,
        "merge": True,
        "penProperty": True,
        "property": True,
        "setAlignment": True,
        "setBackground": True,
        "setBorder": True,
        "setBorderBrush": True,
        "setBorderCollapse": True,
        "setBorderStyle": True,
        "setBottomMargin": True,
        "setCellPadding": True,
        "setCellSpacing": True,
        "setColumnWidthConstraints": True,
        "setColumns": True,
        "setForeground": True,
        "setHeaderRowCount": True,
        "setLayoutDirection": True,
        "setLeftMargin": True,
        "setMargin": True,
        "setObjectIndex": True,
        "setObjectType": True,
        "setPadding": True,
        "setPageBreakPolicy": True,
        "setPosition": True,
        "setRightMargin": True,
        "setTopMargin": True,
        "stringProperty": True,
        "swap": True
    },
    "QtGui.QTouchDevice": {
        "setCapabilities": True,
        "setMaximumTouchPoints": True,
        "setName": True,
        "setType": True
    },
    "QtGui.QTouchEvent": {
        "setAccepted": True,
        "setDevice": True,
        "setTimestamp": True
    },
    "QtGui.QTransform": {
        "mapToPolygon": True,
        "rotate": True,
        "rotateRadians": True,
        "scale": True,
        "setMatrix": True,
        "shear": True,
        "translate": True
    },
    "QtGui.QValidator": {
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "findChild": True,
        "fixup": True,
        "inherits": True,
        "installEventFilter": True,
        "isSignalConnected": True,
        "killTimer": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "setLocale": True,
        "setObjectName": True,
        "setParent": True,
        "setProperty": True,
        "startTimer": True,
        "tr": True,
        "validate": True
    },
    "QtGui.QVector2D": {
        "distanceToLine": True,
        "distanceToPoint": True,
        "setX": True,
        "setY": True
    },
    "QtGui.QVector3D": {
        "distanceToLine": True,
        "distanceToPoint": True,
        "project": True,
        "setX": True,
        "setY": True,
        "setZ": True,
        "unproject": True
    },
    "QtGui.QVector4D": {
        "setW": True,
        "setX": True,
        "setY": True,
        "setZ": True
    },
    "QtGui.QWhatsThisClickedEvent": {
        "setAccepted": True
    },
    "QtGui.QWheelEvent": {
        "setAccepted": True,
        "setTimestamp": True
    },
    "QtGui.QWindow": {
        "alert": True,
        "blockSignals": True,
        "connectNotify": True,
        "disconnectNotify": True,
        "event": True,
        "exposeEvent": True,
        "findChild": True,
        "focusInEvent": True,
        "focusOutEvent": True,
        "hideEvent": True,
        "inherits": True,
        "installEventFilter": True,
        "isAncestorOf": True,
        "isSignalConnected": True,
        "keyPressEvent": True,
        "keyReleaseEvent": True,
        "killTimer": True,
        "mapFromGlobal": True,
        "mapToGlobal": True,
        "mouseDoubleClickEvent": True,
        "mouseMoveEvent": True,
        "mousePressEvent": True,
        "mouseReleaseEvent": True,
        "moveEvent": True,
        "moveToThread": True,
        "property": True,
        "receivers": True,
        "removeEventFilter": True,
        "reportContentOrientationChange": True,
        "resizeEvent": True,
        "setBaseSize": True,
        "setCursor": True,
        "setFilePath": True,
        "setFlag": True,
        "setFlags": True,
        "setFormat": True,
        "setFramePosition": True,
        "setHeight": True,
        "setIcon": True,
        "setKeyboardGrabEnabled": True,
        "setMask": True,
        "setMaximumHeight": True,
        "setMaximumSize": True,
        "setMaximumWidth": True,
        "setMinimumHeight": True,
        "setMinimumSize": True,
        "setMinimumWidth": True,
        "setModality": True,
        "setMouseGrabEnabled": True,
        "setObjectName": True,
        "setOpacity": True,
        "setProperty": True,
        "setScreen": True,
        "setSizeIncrement": True,
        "setSurfaceType": True,
        "setTitle": True,
        "setTransientParent": True,
        "setVisibility": True,
        "setVisible": True,
        "setWidth": True,
        "setWindowState": True,
        "setWindowStates": True,
        "setX": True,
        "setY": True,
        "showEvent": True,
        "startSystemResize": True,
        "startTimer": True,
        "tabletEvent": True,
        "touchEvent": True,
        "tr": True,
        "wheelEvent": True
    },
    "QtGui.QWindowStateChangeEvent": {
        "setAccepted": True
    }
}